      subroutine intrp (cn,imc,ic,jc,fn,imf,if,jf,cx,cy,fx,fy)          
      dimension cn(imc,jc),cx(ic),cy(jc),fn(imf,jf),fx(if),fy(jf)       
c     intrp interpolates data from grid cn to grid fn where the         
c     gridspace cn completely overlies the gridspace fn.                
c     imc= 1st dimension of cn in calling routine                       
c     ic= ending i index on cn                                          
c     jc= ending j index on cn                                          
c     imf= 1st dimension of fn in calling routine                       
c     if= ending i index on fn                                          
c     jf= ending j index on fn                                          
c     cx= distribution of grid points in i direction on cn              
c     cy= distribution of grid points in j direction on cn              
c     fx= distribution of grid points in i direction on fn              
c     fy= distribution of grid points in j direction on fn              
c     the distributions cx and fx (cy and fy) are arbitrary except that 
c     they must be monotonic increasing and consistent with each other. 
c     i.e. if cx(i)=i say, then fx(i) will take on values consistent    
c     with this distribution and determined by the placement of the     
c     points of fn with respect to that of the points of cn.            
c     note that cx(1) should be less than or equal to fx(1)             
c               cy(1) should be less than or equal to fy(1)             
c               cx(ic) should be greater than or equal to fx(ic)        
c               cy(ic) should be greater than or equal to fy(ic)        
c     courtesy of gfdl/noaa ron pacanowski                              
      if(cx(1).le.fx(1).and.cx(ic).ge.fx(if).and.cy(1).le.fy(1).and.    
     1 cy(jc).ge.fy(jf)) go to 10                                       
      write(6,9999)                                                     
      go to 20                                                          
  10  do 12 i=2,ic                                                      
      if(cx(i).ge.cx(i-1)) go to 12                                     
      write (6,9998)                                                    
      go to 20                                                          
  12  continue                                                          
      do 14 j=2,jc                                                      
      if(cy(j).ge.cy(j-1)) go to 14                                     
      write (6,9998)                                                    
      go to 20                                                          
  14  continue                                                          
      do 16 i=2,if                                                      
      if(fx(i).ge.fx(i-1)) go to 16                                     
      write (6,9998)                                                    
      go to 20                                                          
   16 continue                                                          
      do 18 j=2,jf                                                      
      if(fy(j).ge.fy(j-1)) go to 18                                     
      write(6,9998)                                                     
      go to 20                                                          
   18 continue                                                          
      go to 30                                                          
  20  write (6,9994)                                                    
      write (6,9993) (cx(i),i=1,ic)                                     
      write (6,9993) (cy(j),j=1,jc)                                     
      write (6,9993) (fx(i),i=1,if)                                     
      write (6,9993) (fy(j),j=1,jf)                                     
      stop                                                              
   30 continue                                                          
      do 100 j=1,jf                                                     
      do 110 l=1,jc                                                     
      if(cy(l).lt.fy(j)) go to 110                                      
      jn=l-1                                                            
      js=l                                                              
      if(jn.eq.0) js=2                                                  
      if(jn.eq.0) jn=1                                                  
      go to 111                                                         
  110 continue                                                          
  111 continue                                                          
      do 100 i=1,if                                                     
      do 120 l=1,ic                                                     
      if(cx(l).lt.fx(i)) go to 120                                      
      ie=l                                                              
      iw=l-1                                                            
      if(iw.eq.0) ie=2                                                  
      if(iw.eq.0) iw=1                                                  
      go to 121                                                         
  120 continue                                                          
  121 continue                                                          
      a=fx(i)-cx(iw)                                                    
      b=cx(ie)-fx(i)                                                    
      tx=cx(ie)-cx(iw)                                                  
      c=fy(j)-cy(jn)                                                    
      d=cy(js)-fy(j)                                                    
      ty=cy(js)-cy(jn)                                                  
  100 fn(i,j)=(c*(a*cn(ie,js)+b*cn(iw,js))+d*(a*cn(ie,jn)+b*cn(iw,jn))) 
     1 /(tx*ty)                                                         
      return                                                            
 9999 format(1x,'grid interpolated to exceeds grid int from')           
 9998 format(1x,'cx,cy,fx, or fy not monotonic')                        
 9994 format(1x,'cx,cy,fx,fy follow')                                   
 9993 format(1x,10e13.6)                                                
      end                                                               
