      SUBROUTINE DATIMX(NOW)
C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    DATIMX      PROVIDES DATE/TIME INFORMATION
C   PRGMMR: R.E.JONES        ORG: W/NMC42    DATE: 93-11-10
C
C ABSTRACT: DATIMX PROVIDES YOU WITH DATE AND TIME INFORMATION 
C   THAT CAN BE USED BY YOUR PROGRAMS. IT HAS THE SAME OUTPUT
C   AS IBM SUBROUTINE DATIMX IN IBM VS FORTRAN 2.5
C
C PROGRAM HISTORY LOG:
C   93-11-10  R.E.JONES   SAME AS IBM VS FORTRAN VER 2.5
C   96-08-09  R.E.JONES   CORRECTION FOR ELEMENT 10
C   98-01-15  B. VUONG    DISPLAY A WARNING MESSAGE IF ROUTINE IS CALLED
C                         FOR EXECUTION BEFORE JUNE 01, 2001.
C
C USAGE:    CALL DATIMX(NOW)
C   OUTPUT ARGUMENT LIST:  
C     NOW       - INTEGER   ARRAY WITH 14 WORDS
C     
C     ELEMENT   CONTENTS
C
C     1         THE VALUE OF THE CLOCK EXPRESSED AS A POSTITIVE 
C               INTEGER, SET TO 1, SET TO -1 IF INVALID
C     2         MILLISECONDS (0-990) PRECISE TO THE HUNDREDTHS   
C               POSITION, SET TO ZERO ON SGI
C     3         SECONDS (0-59)
C     4         MINUTES (0-59)
C     5         HOUR USING A 24-HOUR CLOCK (0-23)
C     6         DAY OF MONTH  (1-31)
C     7         MONTH OF YEAR (1-12)
C     8         YEAR (4 DIGIT, FOR EXAMPLE: 1993)
C     9         RESERVED. VALUE RETURNED AS -1.
C               WE ARE GOING TO PUT THE JULIAN DAY NUMBER HERE
C     10        HOUR USING 12-HOUR CLOCK (1-12)
C     11        AM/PM INDICATOR. 1 IS RETURNED FOR AM, 2 FOR PM.
C     12        DAY OF WEEK (1-7) BEGINNING WITH SUNDAY
C     13        DAY OF YEAR (1-366)
C     14        YEAR (2 DIGIT, FOR EXAMPLE: 93)
C
C   SUBPROGRAMS CALLED:
C     LIBRARY:
C      SYSTEM  - DATE, CLOCK, SECOND
C       W3LIB  - IW3JDN, W3MSG
C 
C WARNING: THIS SUBROUTINE IS NOT PORTABLE, DATE SUBROUTINES ARE
C          NOT STANDARD FORTRAN 77.
C          THIS SUBROUTINE WILL STOP RUNNING ON JUNE 01, 2001.
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77/90
C   MACHINE:  CRAY
C
C$$$
C
      INTEGER       NOW(14)
      INTEGER       TIME
      INTEGER       DATEK
C
      CHARACTER * 8 ATIME,W3RTINE
      CHARACTER * 8 DATEKK
C
      SAVE
C
      EQUIVALENCE   (TIME,ATIME)
      EQUIVALENCE   (DATEK,DATEKK)
C
C     DISPLAYS A WARNING MESSAGE IF THE ROUTINE
C     IS CALLED FOR EXECUTION BEFORE JUNE 01, 2001.
C     THE ERROR RETURN CODE SET:
C          IERRCODE = 0  IF THE DATE IS BEFORE JUNE 01, 2001
C          IERRCODE = 1  IF THE DATE IS AFTER  JUNE 01, 2001
C
      W3RTINE = 'DATIMX'
      CALL W3MSG(W3RTINE,IERR)
      IF (IERR.EQ.1) RETURN
C
C     GET TIME  (HH:MM:SS) FROM FUNCTION CLOCK
C
      CALL CLOCK(TIME)
C
C     GET DATE  (MM/DD/YY) FROM FUNCTION DATE
C
      CALL DATE(DATEK)
C
C     DECODE DATE INTO MONTH, DAY, YEAR, USE INTERNAL READ
C
      READ(DATEKK,10) NOW(7),NOW(6),NOW(14)
C
      NOW(1)  =  1
C 
C     ATIME IS CHARACTER DATA IN HH:MM:SS FORMAT 
C    
      READ(ATIME,10) NOW(5),NOW(4),NOW(3)
   10 FORMAT ( I2,1X,I2,1X,I2)
C
C     CONVERT 2 DIGIT-YEAR TO 4 DIGIT-YEAR (1921-2020)
C
      IF (NOW(14).LT.21) THEN
        NOW(8) = NOW(14) + 2000
      ELSE
        NOW(8) = NOW(14) + 1900
      END IF
C
      IF (NOW(5).GT.11) THEN
        NOW(11) = 2
        NOW(10) = NOW(5) - 11
      ELSE
        NOW(11) = 1
        NOW(10) = NOW(5) + 1
      END IF
C
C     COMPUTE JULIAN DAY NUMBER  (JAN 1,1960 IS 2,436,935)
C
      JDN    = IW3JDN(NOW(8),NOW(7),NOW(6))
      NOW(9) = JDN
C
C     COMPUTE DAY OF WEEK FROM JULIAN DAY NUMBER
C
      NOW(12) = MOD((JDN + 1),7) + 1
      IYR     = NOW(8)
C
C     COMPUTE DAY OF YEAR (1-366) USING JULIAN DAY NUMBER
C
      NOW(13) = JDN -
     &  (-31739 +1461 * (IYR+4799) / 4 - 3 * ((IYR+4899)/100)/4)
C
C     NO CLOCK FUNCTION TO GET 1000 TH OF SEC, SET TO ZERO ON SGI
C
      SEC1   = SECOND()
      SEC2   = AINT(SEC1)
      NOW(2) = (SEC1 - SEC2) * 1000.0
C
      RETURN
      END
