c    67  1         2         3         4         5         6         7 2
c*
c@***h* ERSST/dati2.upd.situ.v3b.2.f
c*
c* NAME
c*  dati2.upd.situ.v3b.2.f
c*
c* LOCATION
c*  /raid2/ersst/ftn/SST/
c*
c* WARNING 
c* Change destination directories if not running operationally.
c* 
c* PURPOSE
c* To write monthly SST and normalized sampling error variance
c* into ASCII files for distribution.  Only writes out the updated decade.
c*
c* DESCRIPTION
c* The binary files are read and written as intergers (deg c* 100) in
c* ASCII format. Land is -9999. ASCII files are in decadal sets,
c* i.e., 1854 to 1859, 1860-1869....
c*
c* AUTHOR
c*  Thomas M Smith
c*
c* CREATION DATE 
c*  unknown
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  2010 June -Chunying Liu changed from dati2.upd.f to create the other ten year ascii file. 
c*  2009 July -Viva Banzon added comments and put into robodoc format
c*             and also changed paths to versst directory
c*
c* INPUTS
c*    iyr1    ..... latest year
c* 2-deg grids from 1985 onwards
c*    unit 21 ..... sst2d.situ.1985.last.v3b.dat	(updated reconstructed SST)
c*    unit 21 ..... err.norm.map.upd.situ.1985.last.v3b.dat (updated error variance)
c*
c* OUTPUTS
c* 2-deg monthly data in decadal sets; cy1 and cy2 updated by program
c*    unit 51 ..... ersst.v3b.cy1.cy2.asc          (SST)
c*    unit 52 ..... ersst.esd.v3b.cy1.cy2.asc      (standard dev)
c*
c* PARAMETERS
c*    im, jm =xsize and ysize of 2 degree grid
c*
c* VARIABLES
c*	iy1    ..... start of decade to write as ASCII
c*	iy2    ..... end of decade to write as ASCII
c*	a2	= monthly reconstructed SST
c*	e2	= monthly error variance
c*	ierr	= monthly std dev (after taking square root)
c*	isst	= monthly SST *100 (as an integer)
c*
c* TODO
c*   error message if input file read fails
c*
c* SUBPROGRAMS
c*  none
c* 
c* LANGUAGE
c*  Fortran 
c*
c@*****

      parameter (im=180,jm=89)
      real a2(im,jm),e2(im,jm)
      integer isst(im,jm),ierr(im,jm)
      character fm*11,st*3,cy1*4,cy2*4,date*6
      CHARACTER(LEN=60) :: Dir_data
      data fm/'unformatted'/,st/'old'/
      
      CALL getenv("ersst_DAT_DIR", Dir_data)
      Dir_data='/raid/rsad/ersst/data'
c
c      print*, 'Input the processing year, month'
      read(5,*) iyrl,imoend
      print*, 'The processing year, month', iyrl, imoend
      
      iyrp=iyrl+1
      mm=mod(iyrp,10)
      iy2=iyrl-10
      if(mm.gt.0) iy2=iyrl-mm
      iy1=iy2-9
      write(cy1,'(i4)') iy1
      write(cy2,'(i4)') iy2
      print*, 'Writing to file '//cy1//'.'//cy2//'.'
      open(51,file=adjustl(adjustr(Dir_data) //
     2     '/output/ersst.v3b.'//cy1//'.'//cy2//'.asc'))
      open(52,file=adjustl(adjustr(Dir_data) //
     2   '/output/ersst.esd.v3b.'//cy1//'.'//cy2//'.asc'))
c
      
      open(22,status=st,form=fm, file=adjustl(adjustr(Dir_data) //
     2  '/sst/err.norm.map.upd.situ.1985.last.v3b.dat'))
     
      do iy=1985,iy1-1    
       do mon=1,12
       
        write (date,'(i4.4,i2.2)')  iy,mon

        open(21,form=fm,status=st,
     1        file= adjustl(adjustr(Dir_data)//
     2     '/output/sst2d.situ.' // date // '.v3b.dat'))

       
        read(21,end=100,err=100) a2
        read(22,end=100,err=100) e2
       end do
      end do
      
c
      do iy=iy1,iy2
       do mon=1,12
       
        write (date,'(i4.4,i2.2)')  iy,mon
	
	PRINT*, adjustl(adjustr(Dir_data)//
     2     '/output/sst2d.situ.' // date // '.v3b.dat')

        open(21,form=fm,status=st,
     1        file= adjustl(adjustr(Dir_data)//
     2     '/output/sst2d.situ.' // date // '.v3b.dat'))
       
        read(21,end=10,err=10) a2
        read(22,end=10,err=10) e2
	
	e2=sqrt(e2)
	
        iylst=iy
        molst=mon
	
	
        do 2 j=1,jm
        do 2 i=1,im
         isst(i,j)=nint(100.0*a2(i,j))
         if(a2(i,j).lt.-990.) isst(i,j)=-9999
         ierr(i,j)=nint(100.0*e2(i,j))
         if(e2(i,j).lt.-990.) ierr(i,j)=-9999
 2      continue
c
        do i=1,im
         write(51,81) (isst(i,j),j=1,jm)
         write(52,81) (ierr(i,j),j=1,jm)
        end do
	
       end do
      end do
c
10    print*, 'Last Time Done: ',molst,', ',iylst

      if(iylst.lt.iy2)then
       print*, 'RED-STOP! Problem in dati2.upd.f start year > end year'
       call exit(100)
      endif
      print*, 'GREEN-SUCCESSFUL! dati2.upd.f'
      call exit(1)

100    print*, 'RED-STOP! data input problem in dati2.upd.f'
       call exit(100)
      
   81 format (90i6)
      end
