c    67  1         2         3         4         5         6         7 2
c@***h*  ERSST/err.norm.map2.upd.situ.v3b.f
c*
c* NAME 
c*   err.norm.map2.upd.situ.v3b.f
c*
c* LOCATION
c*  $ersst_SRC_DIR/err.norm.map2.upd.situ.v3b.f
c*
c* WARNING
c*  Change destination directories if not running operationally.
c*  Operationally, only the more recent years (from 1985) are updated
c*  and the historical period is left alone.
c*  Some variable names in the main program are used 
c*  in subroutines for temporary variables.
c* 
c* PURPOSE
c*  To compute the normalized sampling error variance maps for ERSST
c*  from 1985 onwards
c* 
c* DESCRIPTION
c*  The total sampling error variance is estimated by computing the LF 
c*  and HF variances on the 2 deg grid (88S-88N by 0E-358E).The two 
c*  variances are then normalized and summed to produce the total.
c*  The LF sampling error variance is modeled from the annual sampling
c*  over 25-deg areas (5 X 5 deg grid) and the average damping of the error 
c*  with sampling. The "reference" LF sampling variance is as static file
c*  created from model SST anomalies (1861-2000) that are first low-pass 
c*  filtered .  The model is the GFDL Coupled Global Climate Model used
c*  for tuning and testing the reconstruction.  The CGCM results were 
c*  averaged to 5 deg grid to match and evaluate the merged land -ocean
c*  product.  To evaluate the LF component on the 2-deg grid, the LF
c*  reference field is first regridded to 2 deg.  The HF sampling error
c*  variance is computed from the variance associated with each mode and
c*  how many modes are resolved. A residual HF error is added for variations
c*  never resolved.  The reference HF variance field is computed from OI v2
c*  monthly SST anomalies 1982-2005)
c*  See: Smith and Reynolds (2004) for Error analysis details
c*
c* AUTHOR
c*  Thomas M Smith
c*
c* CREATION DATE
c*  February 2007
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* PROGRAM HISTORY
c*  03/03/2010 - C. Liu hardcoding
c*  2009 Sep: V. Banzon added comments as part of documentation effort
c*  Jan 2007  - Chunying Liu modified the code to run at NCDC 
c*
c* INPUTS
c*   passed interactively or by script
c*    	iydo ........ user-specified year to do
c*	mondo ....... user-specified month to do
c*   land mask (land = 0, sea = 1)
c*      unit 30 ..... $ersst_DAT_DIR/input/static/mask2d.dat
c*	unit 21 ..... $ersst_DAT_DIR/output/anom2d/sst2d.ano.situ.YYYYMM.v3b.dat
c*	unit 21 ..... $ersst_DAT_DIR/input/static/varhf.ssta.dat (2-d high freq variances, from T. Smith)
c*              computed from detrended OI v2 monthly SST anomalies 1982-2005)
c*	unit 21 ..... $ersst_DAT_DIR/input/static/lfvar5d.mrg.dat (5-d low freq variances, from T. Smith)
c*              computed from low pass filtered CGCM SSTanomalies 1861-2000 )
c*   program updates cyr
c*	unit 23 ..... $ersst_DAT_DIR/inter/situmerg/ssta.merg.situ.mon.v3b.cyr.dat
c*
c* OUTPUTS 
c*	unit 51 ..... $ersst_DAT_DIR/output/err2d/err.norm.map.upd.situ.1985.last.v3b.dat
c*	
c* PARAMETERS
c*	im, jm		= x- and y-size of 2 deg grid
c*	ehmin		= 0.05
c*	nyav		= size of averaging window in years
c*	ntm		= size of averaging window in months
c*
c* VARIABLES
c*  Arrays that are on a 5 deg grid:
c*	vl5	= CGCM model full LF sampling variance
c*
c*  Arrays that are on the 2 deg grid:
c*	vlf	= regridded LF variance returned by int5t2
c*	vhf	= variance estimate for HF analysis
c*	a2	= reconstructed analysis monthly SST anomaly
c*		= HF std dev returned by getsda
c*	ac	= monthly merged buoy/ship SST anomaly
c*		= updated by lferr to hold annual LF sampling variance
c*
c*  Arrays on a 2 deg grid by ntm months (9 year window)
c*	ert	= monthly SST anomalies used in HF computation
c*		= retuned by detrnd as detrended monthly anomalies
c*
c*  Arrays containing a year of monthly 2 deg grids
c*	sstp	= monthly merged buoy/ship anomalies for 1 year

c* SUBPROGRAMS
c*	int5t2
c*	lferr
c* 	detrnd
c* 	getsda
c*
c* TODO
c*
c* LANGUAGE
c*  Fortran
c*
c@****
c* =======================END MAIN HEADER=======================

      parameter(ehmin=0.05,nyav=9,ntm=nyav*12,im=180,jm=89)
      real a2(im,jm),ac(im,jm),sstp(im,jm,12)
      real ert(im,jm,ntm),vhf(im,jm),vlf(im,jm),vl5(72,36)
      integer mask(im,jm)
      character fm*11,cyr*4,st*3,date*6
      CHARACTER(LEN=80) :: Dir_data,fname
      data fm/'unformatted'/, st/'old'/
      
      print*, 'Input the year to do:'
      read(5,*) iydo
      print*, 'Input the month to do:'
      read(5,*) mondo
      
      CALL getenv("ersst_DAT_DIR", Dir_data)
c
c open output file

       fname=adjustl(adjustr(Dir_data) //
     &   '/output/err2d/err.norm.map.upd.situ.1985.last.v3b.dat')
      print*,'Opening '//fname
      open(51,form=fm, err=100,file=fname) 
c
c* Read in the variances needed to estimate errors.

       fname=adjustl(adjustr(Dir_data) //
     &   '/input/static/varhf.ssta.dat')
      print*,'Opening '//fname
      open(21,form=fm, status=st,err=100,file=fname)
      read(21,end=100,err=100) vhf
      close(21)
c      print*,'Got HF variance'

c Read 5-d LF variance      

       fname=adjustl(adjustr(Dir_data) //
     &   '/input/static/lfvar5d.mrg.dat')
      print*,'Opening '//fname
      open(21,form=fm, status=st,err=100,file=fname)
c      print*,'Got 5-d LF variance'

      read(21,end=100,err=100) vl5
      call int5t2(vl5,vlf)
      close(21)
c
c* Read in the 2-d mask (land = 0, sea = 1).

       fname=adjustl(adjustr(Dir_data) //
     &     '/input/static/mask2d.dat')
      print*,'Opening '//fname
      open(30,form=fm,status=st, err=100,file=fname)

      read(30,end=100,err=100) mask
      close(30)
c      print*,'Got 2d land mask'
c
c* Initalize the ert array with anomalies
      print*, 'Reading analysis anomalies'
      do iy=1,nyav
      
         do mon=1,12
            nt=12*(iy-1)+mon
            write (date,'(i4.4,i2.2)')  iy+1984,mon
            fname=adjustl(adjustr(Dir_data) //
     &         '/output/anom2d/sst2d.ano.situ.'//date//'.v3b.dat')
c            print*,'Opening '//fname
            open(21,form=fm, status=st,err=100,file=fname)

            read(21,end=100,err=100) a2
            do j=1,jm
               do i=1,im
                 ert(i,j,nt)=a2(i,j)
               end do
            end do 
         end do
      end do
     
c
      do iy=1985,iydo
c* First compute the LF sampling error variance from the updates
c* sampling.  This is computed once/year.
         write(cyr,'(i4)') iy
         print*,'Reading in situ anomalies to compute LF var'
         fname=adjustl(adjustr(Dir_data) //
     &     '/inter/situmerg/ssta.merg.situ.mon.v3b.'//cyr//'.dat')
c         print*,'Opening '//fname
         open(23,form=fm,status=st, err=100,file=fname)

          monm=12
          if(iy.eq.iydo)monm=mondo
          do mon=1,monm
             read(23,end=100,err=100) ac
             do 4 j=1,jm
             do 4 i=1,im
                sstp(i,j,mon)=ac(i,j)
 4           continue
          end do

c* Get the spatial and annual sampling %.
         call lferr(sstp,mask,vlf,ac)
         do mon=1,monm
c* Next compute the HF sampling error variance.  This is computed
c* once/month.
c* The anoms are read again but position is shifted.
c* The purpose is to  populate the detrend array
            write (date,'(i4.4,i2.2)')  iy,mon
         print*,'Reading analysis anoms to compute HF var'
         fname=adjustl(adjustr(Dir_data) //
     &     '/output/anom2d/sst2d.ano.situ.'//date//'.v3b.dat')        
c         PRINT*,'Opening '//fname
         open(21,form=fm, status=st,err=100,file=fname)

         read(21,end=100,err=100) a2
         do j=1,jm
            do i=1,im
               do nt=1,ntm-1
                  ert(i,j,nt)=ert(i,j,nt+1)
               end do
               ert(i,j,ntm)=a2(i,j)
            end do
         end do
         call detrnd(mask,ert,ntm)
         call getsda(mask,ert,ntm,a2)
c* Combine and normalize the error variance.
         do 6 j=1,jm
            lat=2*j-90
         do 6 i=1,im
            if(mask(i,j).eq.1)then
               shf=sqrt(vhf(i,j))
               dd=(shf-a2(i,j))
               dck=ehmin*shf
               if(dd.lt.dck)   dd=dck
               if(dd.gt.shf)   dd=shf
               tmp=dd*dd+ac(i,j)
               a2(i,j)=tmp/(vlf(i,j)+vhf(i,j))
               if(a2(i,j).gt.1.0) a2(i,j)=1.0
               if(lat.le.-78)     a2(i,j)=-999.9
            else
               a2(i,j)=-999.9
            endif
 6       continue
         write(51) a2
       end do
       print*, 'Finished year ',iy
      end do
      
      print*, 'GREEN-SUCCESSFUL! Code successful'
      call exit(1)
 100  print*, 'RED-STOP! sst2d.f data problem'
      call exit(100)
      end
c
c@***s* ERSST/err.norm.map2.upd.situ.v3b.f/int5t2
c*
c* NAME
c*   int5t2
c*
c* LOCATION
c*  inside $ersst_SRC_DIR/err.norm.map2.upd.situ.v3b.f
c*
c* WARNING
c*  Assumes input data is on a 5 deg Grid: 2.5E-357.5E, 87.5S-87.5N.
c*
c* PURPOSE
c*  To interpolate from a 5-deg grid to a 2-deg grid.
c*
c* DESCRIPTION
c*  The 5 deg data is regridded to 2 deg by first creating a 1 deg grid
c*  consisting of each 5 deg value repeated 5 times on an axis.  The
c*  1 deg values are then averaged every 2 deg.
c*  The 5-d grid: 2.5E-357.5E, 87.5S-87.5N.
c*  The 2-d grid:   0E-358E,     88S-88N.
c*  Use the intermediate 1-d grid: 0.5E-359.5E, 89.5S-89.5N.
c*
c* AUTHOR
c*  Thomas M. Smith
c*
c* CREATION DATE
c*  Feb 2007 (presumed)
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* PROGRAM HISTORY
c*  2009 Oct: V. Banzon added comments as part of documentation effort
c*
c* INPUTS
c*   a5	= data on 5 deg grid 
c*
c* OUTPUTS 
c*   a2	= data on 2 deg grid 
c*
c* PARAMETERS
c*   im5,jm5	= xsize, ysize of 5 deg grid
c*   im2,jm2	= xsize, ysize of 2 deg grid
c*   im1,jm1	= xsize, ysize of 1 deg grid
c*
c* VARIABLES
c*   a1	= data on 1 deg grid 
c*   jls,jle	= start and end y-index of 1 deg boxes to match single 5 deg box
c*   ils,ile	= start and end x-index of 1 deg boxes to match single 5 deg box
c*   jls,jle, ils, ile are used again to for 1 deg boxes to average to 2 deg
c*
c* TODO
c*  Put this in a subroutine library
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
c
      subroutine int5t2(a5,a2)
      parameter(im5=72,jm5=36,im2=180,jm2=89,im1=360,jm1=180)
      real a1(im1,jm1),a2(im2,jm2),a5(im5,jm5)
c* First fill the 1-d grid, then average to the 2-d grid.
      do j5=1,jm5
       j1e=5*j5
       j1s=j1e-4
       do i5=1,im5
        i1e=5*i5
        i1s=i1e-4
        do 2 j1=j1s,j1e
        do 2 i1=i1s,i1e
         a1(i1,j1)=a5(i5,j5)
 2      continue
       end do
      end do
c
      do j2=1,jm2
       j1s=2*j2
       j1e=j1s+1
       do i2=1,im2
        i1s=2*i2-2
        i1e=i1s+1
        su1=0.0
        su2=0.0
        do i1=i1s,i1e
         id=i1
         if(id.eq.0) id=im1
         do j1=j1s,j1e
          if(a1(id,j1).gt.-990.)then
           su1=su1+a1(id,j1)
           su2=su2+1.0
          endif
         end do
        end do
        a2(i2,j2)=-999.9
        if(su2.gt.0.0) a2(i2,j2)=su1/su2
       end do
      end do
      return
      end
c-------------------------------------------------------------------------
c@***s* ERSST/err.norm.map2.upd.situ.v3b.f/lferr
c*
c* NAME
c*   lferr
c*
c* LOCATION
c*  inside $ersst_SRC_DIR/err.norm.map2.upd.situ.v3b.f
c*
c* WARNING
c*  Assumes input data is on a 2 deg Grid: 0E-358E, 88S-88N.
c*  sstp is also a variable in the main program
c*
c* PURPOSE
c*  To compute the LF sampling error.
c*
c* DESCRIPTION
c*  The LF sampling error is computed based on the availability of in situ data.
c*  A static file containing the full LF variance from the GFDL CGCM 
c*  anomalies(1861-2000) is required.
c*  First the adequacy of the monthly spatial sampling within the 
c*  26 deg box about the center is checked (<3%  are considered undersampled).  
c*  This monthly data count is then applied to check the annual sampling. 
c*  For each gridpoint, the full LF error from the models is used but
c*  it is adjusted using a factor df.  
c*    when months <3, df is forced to be 0.9
c*    when 2<month<12, df exponentially approaches zero   
c*    with 12 months sampling, df is force to 0.
c*  Thus the error is forced to zero when 12 months of data are available.
c*  Theoretically, df =1 when there is no data, but instead this boxes 
c*  are already flagged with a -999.99.
c*
c* AUTHOR
c*  Thomas M. Smith
c*
c* CREATION DATE
c*  Feb 2007
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* PROGRAM HISTORY
c*  2009 Oct: V. Banzon added comments as part of documentation effort
c*
c* INPUTS
c*  all on a 2 deg grid:
c*	sstp	= in situ SST anomalies
c*	mask	= sea mask (land=0, sea=1)
c* 	vlf 	= regridded "reference" low frequency variance from detrended OI
c*
c* OUTPUTS 
c* 	elf 	= low frequency analysis error
c*
c* PARAMETERS
c*	im,jm	= xsize, ysize of 2 deg grid
c*
c* VARIABLES
c*	cj	= area weights for 2 deg grid
c*    sstp	= first contains the in situ SST anomalies
c*        	= then contains the sampling adequacy score from elf
c*                (1.0=OK, -999.9=inadequate)
c* 	vlf	= full low frequency variance from models
c* 	elf	= first contains the sampling adequacy score
c*        	= on output, contains the low frequency analysis error
c*
c* TODO
c*  Put this in a subroutine library
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
      subroutine lferr(sstp,mask,vlf,elf)
      parameter(im=180,jm=89)
      real sstp(im,jm,12),cj(jm),vlf(im,jm),elf(im,jm)
      integer mask(im,jm)
      data cj/0.03,0.07,0.10,0.14,0.17,0.21,0.24,0.28,0.31,0.34,
     2        0.37,0.41,0.44,0.47,0.50,0.53,0.56,0.59,0.62,0.64,
     3        0.67,0.69,0.72,0.74,0.77,0.79,0.81,0.83,0.85,0.87,
     4        0.88,0.90,0.91,0.93,0.94,0.95,0.96,0.97,0.98,0.98,
     5        0.99,0.99,1.00,1.00,1.00,1.00,1.00,0.99,0.99,0.98,
     6        0.98,0.97,0.96,0.95,0.94,0.93,0.91,0.90,0.88,0.87,
     7        0.85,0.83,0.81,0.79,0.77,0.74,0.72,0.69,0.67,0.64,
     8        0.62,0.59,0.56,0.53,0.50,0.47,0.44,0.41,0.37,0.34,
     9        0.31,0.28,0.24,0.21,0.17,0.14,0.10,0.07,0.03/
c* First the spatial averaging over 26-deg spatial moving areas.
      do mon=1,12
       do 2 j0=1,jm
       js=j0-6
       if(js.lt.1) js=1
       je=j0+6
       if(je.gt.jm)je=jm
       do 2 i0=1,im
        elf(i0,j0)=-999.9
        if(mask(i0,j0).eq.0) go to 2
        elf(i0,j0)=0.0
        su1=0.0
        su2=0.0
        sus=0.0
        sut=0.0
        do i=i0-6,i0+6
         id=i
         if(i.lt.1) id=i+im
         if(i.gt.im)id=i-im
         do j=js,je
          ww=cj(j)*real(mask(id,j))
          sut=sut+ww
          if(sstp(id,j,mon).gt.-990.) sus=sus+ww
         end do
        end do
c* Require 3% spatial sampling (about 5 superobs).
        fsa=sus/sut
        if(fsa.ge.0.03)  elf(i0,j0)=1.0
 2     continue
       do 4 j=1,jm
       do 4 i=1,im
        sstp(i,j,mon)=elf(i,j)
 4     continue
      end do
c* Next annual sampling of these.
      do 6 j=1,jm
      do 6 i=1,im
       elf(i,j)=-999.9
       if(mask(i,j).eq.0) go to 6
       su1=0.0
       do mon=1,12
        if(sstp(i,j,mon).gt.-990.) su1=su1+sstp(i,j,mon)/12.0
       end do
       arg=su1/0.4
       df=exp(-arg)
       if(su1.le.0.2)then
        if(df.lt.0.9) df=0.9
       else
        if(su1.gt.0.9) df=0.0
       endif
       elf(i,j)=vlf(i,j)*df
 6    continue
      return
      end
c--------------------------------------------------------------
c@***s* ERSST/err.norm.map2.upd.situ.v3b.f/detrnd
c*
c* NAME
c*   detrnd
c*
c* LOCATION
c*  inside $ersst_SRC_DIR/err.norm.map2.upd.situ.v3b.f
c*
c* WARNING
c*  Assumes input data is on a 2 deg Grid: 0E-358E, 88S-88N.
c*
c* PURPOSE
c*  To detrend the monthly data.
c*
c* DESCRIPTION
c*  The monthly reconstructed anomalies are detrended using a
c*  multi-year averaging window (specified in Main program).
c* The temporal correlation is computed using a least squares
c* fit using only data within the time window. 
c*  The detrended anomalies are returned.
c*
c* AUTHOR
c*  Thomas M. Smith
c*
c* CREATION DATE
c*  Feb 2007
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* PROGRAM HISTORY
c*  2009 Oct: V. Banzon added comments as part of documentation effort
c*
c* INPUTS
c*	at	= reconstructed SST anomalies on 2 deg grid
c*	mask	= sea mask (land=0, sea=1)
c*	ntm	= averaging window size in months
c*
c* RETURNED VALUE 
c* 	at 	= detrended anomalies
c*
c* PARAMETERS
c*	im,jm	= xsize, ysize of 2 deg grid
c*
c* VARIABLES
c*	x1	= sum of x
c* 	x2 	= sum of squared x
c* 	y1 	= sum of y (anomalies)
c*	rn	= number of valid SST data (real)
c*	vx	= variance of x
c*	cx	= covariance
c*	sl	= slope
c*	b0	= intercept
c*	fl	= linear fit
c*
c* TODO
c*  Put this in a subroutine library
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
      subroutine detrnd(mask,at,ntm)
      parameter(im=180,jm=89)
      real at(im,jm,*)
      integer mask(im,jm)
      do 4 j=1,jm
      do 4 i=1,im
       if(mask(i,j).eq.0) go to 4
       x1=0.0
       x2=0.0
       y1=0.0
       xy=0.0
       kn=0
       do n=1,ntm
        if(at(i,j,n).gt.-990.)then
         x=real(n)
         y=at(i,j,n)
         x1=x1+x
         x2=x2+x*x
         y1=y1+y
         xy=xy+x*y
         kn=kn+1
        endif
       end do
       rn=real(kn)
       vx=x2-x1*x1/rn
       cv=xy-x1*y1/rn
       sl=cv/vx
       b0=(y1-sl*x1)/rn
       do n=1,ntm
        if(at(i,j,n).gt.-990.)then
         fl=b0+sl*real(n)
         at(i,j,n)=at(i,j,n)-fl
        endif
       end do
 4    continue
      return
      end
c---------------------------------------------------------------------------
c@***s* ERSST/err.norm.map2.upd.situ.v3b.f/getsda
c*
c* NAME
c*   getsda
c*
c* LOCATION
c*  inside $ersst_SRC_DIR/err.norm.map2.upd.situ.v3b.f
c*
c* WARNING
c*  Assumes input data is on a 2 deg Grid: 0E-358E, 88S-88N.
c*
c* PURPOSE
c*  To compute the all-month s.d. from the detrended data.
c*
c* DESCRIPTION
c*  This is a straightforward standard deviation calculation.  The
c*  variance is computed from the sum squared and the sum of squares.
c*  The square root of the variance is the standard deviation.
c*
c* AUTHOR
c*  Thomas M. Smith
c*
c* CREATION DATE
c*  Feb 2007
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* PROGRAM HISTORY
c*  2009 Oct: V. Banzon added comments as part of documentation effort
c*
c* INPUTS
c*	at	= in situ SST anomalies on 2 deg grid
c*	mask	= sea mask (land=0, sea=1)
c*
c* OUTPUTS 
c* 	sd 	= standard deviation
c*
c* PARAMETERS
c*	im,jm	= xsize, ysize of 2 deg grid
c*	vvmin	= floor limit for variance 
c*
c* VARIABLES
c*	s1	= sum
c* 	s2 	= sum of squarese
c* 	kn 	= number of valid data (interger)
c*	rn  	= number of valid data (real)
c*
c* TODO
c*  Put this in a subroutine library
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****

      subroutine getsda(mask,at,ntd,sd)
      parameter(im=180,jm=89,vmin=0.0001)
      real sd(im,jm),at(im,jm,*)
      integer mask(im,jm)
      do 4 j=1,jm
      do 4 i=1,im
       sd(i,j)=-999.9
       if(mask(i,j).eq.0) go to 4
       s1=0.0
       s2=0.0
       kn=0
       do n=1,ntd
        s1=s1+at(i,j,n)
        s2=s2+at(i,j,n)*at(i,j,n)
        kn=kn+1
       end do
       rn=real(kn)
       s1=s1/rn
       s2=s2/rn
       vv=s2-s1*s1
       if(vv.lt.vmin) vv=vmin
       sd(i,j)=sqrt(vv)
 4    continue
      return
      end
