c*
c@***h* ERSST/ersst_netcdf.situ.v3b.f
c*
c* NAME
c*  ersst_netcdf.situ.v3b.f
c*
c* LOCATION
c*  $ersst_SRC_DIR/
c*
c* WARNING 
c* Change destination directories if not running operationally.
c* 
c* PURPOSE
c* To write monthly SST, anomalies and normalized sampling error variance
c* into netCDF format for distribution.  Only writes out the updated year.
c*
c* DESCRIPTION
c* The program reads the input files for SST, anomalies and error variance,
c* and skips to most recent month. Values are converted to integer (deg C*100)
c* and written to the appropriate location in the netCDF file.
c*
c* AUTHOR
c*  presumably Thomas M Smith
c*
c* CREATION DATE 
c*  unknown
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  undated   -from dati2.upd.f
c*  2009 Sep -Viva Banzon added comments and put into robodoc format
c*          
c*
c* INPUTS
c*	iydo    ..... year to update
c*	mondo   ..... month to update
c*
c* Updated 2-deg grids from 1985 onwards
c*    unit 21 ..... sst2d.situ.YYYYMM.v3b.dat    (reconstructed SST)
c*    unit 22 ..... sst2d.ano.situ.YYYYMM.v3b.dat  (reconstructed anomalies)
c*    unit 23 ..... err.norm.map.upd.situ.1985.last.v3b.dat (error variance)
c*
c* OUTPUTS
c* 2-deg monthly data in decadal sets; cy1 and cy2 updated by program
c*    netcdfpath ..... $ersst_DAT_DIR/output/netcdf/ersst.YYYYMM.nc
c*                     the subroutine actually writes the SST and error
c*                     and main passes the name
c*
c* PARAMETERS
c*     imx, imy	= xsize and ysize of 2 degree grid
c*	iyrref		= reference year (beginning of entire time series)
c*	imonref	= reference month (beginning of entire time series)
c*	idayref	= reference day (beginning of entire time series)
c*	undef	 	= no data (-999.9) in input file
c*	undef2		= no data (-999) in output file
c*
c* VARIABLES
c*	iy1    ..... start of decade to write as ASCII
c*	iy2    ..... end of decade to write as ASCII
c*	sst		= monthly reconstructed SST
c*	anom		= monthly reconstructed anomaly
c*	err		= monthly error variance
c*	sdate		= date string in yyyymmdd
c*	cdate		= date string in yyyy-mm-dd
c*	refdate	= reference date as yyyy-mm-dd
c*	cnetdate	= to do date as yyyy-mm-dd
c*	path		= input path
c*	cdfpath	= output path (updated by year month)
c*	isst		= integerized monthly reconstructed SST * 100
c*	ianom		= integerized monthly reconstructed anomaly * 100
c*	ierr		= integerized monthly reconstructed std dev * 100
c*
c* TODO
c*   error message if input file read fails
c*   the check_err90 subroutine is never called
c*
c* SUBPROGRAMS
c* timer
c* $ersst_SRC_DIR/put_cdf_ersst_situ_v3b.f       
c* $ersst_SRC_DIR/check_err90.f
c* $ersst_LIB_DIR/w3lib/iw3jdn.f
c* $ersst_LIB_DIR/maxmin.f 
c* $ersst_LIB_DIR/imaxmin2.f
c* 
c* LANGUAGE
c*  Fortran 
c*
c@*****

       PARAMETER (imx=180,imy=89) 
       real sst(imx,imy),anom(imx,imy),err(imx,imy)
       integer*2 isst(imx,imy),ianom(imx,imy),ierr(imx,imy)        
       integer*2 undef2            
       character*100 cdfpath,date*6
       character sdate*8,cdate*10, refdate*99,cnetdate*10
       character fm*11,st*3
       data fm/'unformatted'/,st/'old'/
       CHARACTER(LEN=80) :: Dir_data,fname
       logical lexist       
	
       call timer (itime1)
       
       CALL getenv("ersst_DAT_DIR", Dir_data)
c       print*,Dir_data
       
       
       read(5,*) iydo
       print*, 'The processing year:',iydo
      
       read(5,*) mondo 
       print*, 'The processing mondo:',mondo
      
       read*, netiyr,netmon,netday
c       print*, 'NetCDF create year, month,day: ', netiyr,netmon,netday
       
       write (cnetdate,'(i4.4,a1,i2.2,a1,i2.2)') 
     1        netiyr,'-',netmon,'-',netday 
       print*,'NetCDF create date: ',cnetdate
      
      ida=15   
       
       IYRref=1854	 !reference year
       IMONref=1		!January
       IDAYref=15		
       write (refdate,'(i4.4,a1,i2.2,a1,i2.2)') 
     &			IYRref,'-',IMONref,'-',IDAYref
       print*,'refdate=',refdate
       
        write (date,'(i4.4,i2.2)')  iydo,mondo
        cdfpath=adjustl(adjustr(Dir_data)//
     &  '/output/netcdf/ersst.'//date//'.nc')
        print*,'cdfpath=',cdfpath
              
        fname=adjustl(adjustr(Dir_data) //
     &   '/output/sst2d/sst2d.situ.' // date // '.v3b.dat')
c       print*, 'Opening: '//fname
       open(21,form=fm,status=st,err=100,file=fname)
     
        fname=adjustl(adjustr(Dir_data) //
     &   '/output/anom2d/sst2d.ano.situ.' // date // '.v3b.dat')
c         print*, 'Opening: '//fname
         open(22,form=fm,status=st,err=100,file=fname)

        fname=adjustl(adjustr(Dir_data) //
     &  '/output/err2d/err.norm.map.upd.situ.1985.last.v3b.dat')
c         print*, 'Opening: '//fname
         open(23,form=fm,status=st,err=100,file=fname)

        undef = -999.9
        undef2 = -999
     
        do iy=1985,iydo-1
         do mon=1,12
	     read(23,end=100,err=100) err
         end do
        end do
c
       do iy=iydo,iydo
         do mon=1,mondo
	    read(23,end=100,err=100) err
        
	   if(mon .eq.mondo)then
	
	       read(21,end=100,err=100) sst
               read(22,end=100,err=100) anom
	
	       call maxmin (sst,imx,imy,undef,'sst     ')
               call maxmin (anom,imx,imy,undef,'anom    ')
               call maxmin (err,imx,imy,undef,'err     ') 
	
               WHERE (sst(:,:) == undef)        
                  isst(:,:) = undef2
               ELSEWHERE       
                  isst(:,:)= nint(100.*sst(:,:))
               END WHERE
	
	       WHERE (anom(:,:) == undef)        
                  ianom(:,:) = undef2
               ELSEWHERE       
                  ianom(:,:)= nint(100.*anom(:,:))
               END WHERE	
	
               WHERE (err(:,:) == undef)        
                  ierr(:,:) = undef2
               ELSEWHERE
	          err(:,:)=sqrt(err(:,:))       
                  ierr(:,:)= nint(100.*err(:,:))
               END WHERE
	
	       call imaxmin2 (isst,imx,imy,undef2,'isst    ')
               call imaxmin2 (ianom,imx,imy,undef2,'ianom    ')
               call imaxmin2 (ierr,imx,imy,undef2,'ierr     ')
	
               do j=10,10
                  write(*,101)(isst(i,j),i=80,85)
                  write(*,101)(ianom(i,j),i=80,85)
                  write(*,101)(ierr(i,j),i=80,85)
               enddo
101           format(1x,6i10)

           endif
	
          enddo
       enddo
       
       
       write (sdate,'(i4.4,2i2.2)') iydo,mondo,ida
c       print*,'sdate=',sdate
              
       write (cdate,'(i4.4,a1,i2.2,a1,i2.2)') iydo,'-',mondo,'-',ida
c       print*,cdate 
            
      
c  write to netcdf
c  day number from the reference day
       iday_num = iw3jdn(iydo,mondo,ida)-IW3JDN(IYRref,IMONref,IDAYref)
                
       call put_cdf_ersst_situ_v3b
     2 (cdfpath,isst,ianom,ierr,cnetdate,cdate,refdate,iday_num)
	
       call timer (itime2)
       tsec = float(itime2-itime1)*.01
       print *,tsec,' seconds for complete NetCDF code'
       
       print*, 'GREEN-SUCCESSFUL! ersst_netcdf.situ.v3b.f'
       call exit(1)

100    print*, 'RED-STOP! data input problem in ersst_netcdf.situ.v3b.f'
       call exit(100)
       
       end
       
       include  'put_cdf_ersst_situ_v3b.f'    
       include  'check_err90.f'
       include  'iw3jdn.f'
       include  'maxmin.f' 
       include  'imaxmin2.f'
      
