c    67  1         2         3         4         5         6         7 2
c@***h*  ERSST/hfsst.situ.v3b.f
c* 
c* NAME
c*  hfsst.situ.v3b.f
c*
c* LOCATION
c*  $ersst_SRC_DIR/hfsst.situ.v3b.f
c*
c* WARNING
c* Change destination directories if not running operationally.
c* Operationally, only the more recent years (from 1985) are updated
c* and the historical period is left alone.
c* Some variable names in the main program are used 
c* in subroutines for temporary variables.
c* 
c* PURPOSE
c* To produce an updated high-frequency anomaly analysis using
c* updated merged SST anomalies for the period from 1985 to 
c* the user-specified year-month.  
c* 
c* DESCRIPTION
c* The high frequency (HF) analysis of the SST anomalies post-1984 is performed 
c* on a 2-deg grid (lon: 0, 2E, ...2W)  (lat: 88S,..,0, ...88N). Monthly analysis
c* is performed on the HF anomaly residuals (SST anomalies minus the LF anomalies).
c* For the quality screening prior to regression, results from the Optimal 
c* Interpolation (OI) analysis are used.  The OI anomalies and  counts for a 
c* reference period are used to compute std. dev. and variance. The screening
c* is meant to exclude poor quality data from the analysis. EOT modes used come from
c* a separate one-time analysis by T. Smith. For these updates only 1 month of anomalies
c* is used, because of the dense sampling. For each month, data in a three-month time
c* window are examined. At every location, if SST from the center month is available 
c* then that is used.  If not then SST from the adjacent months is used if available,
c* averaging if both adjacent months have an observation.  Screening regression is used
c* to eliminate under-sampled modes from the analysis. Modes weights are written out, 
c* with under-sampled modes flagged as missing (-999.9).
c* Note:  The regression analysis is not performed on a grid since correlations cannot
c* be computed with missing data
c*
c* AUTHOR
c*  Thomas M. Smith
c*
c* CREATION DATE
c*  December 2006
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* PROGRAM HISTORY
c* 2010 March - Chunying Liu hardcoding.
c* 2009 Sep: V. Banzon added comments as part of documentation effort
c* 2008 Apr: C.Liu adapted from Tom's v3b
c* 	Analysis uses 3 months of anomalies.dat3 dat3 dat3 dat3
c* 	Read in the 2-d mask (land = 0, sea = 1)
c* 	Compute the OI S.D. for checking, multiply it by fs
c* 	sdf(i,j)=fs*sqrt(vv)
c* 	if(sdf(i,j).lt.scm) sdf(i,j)=scm
c* July 2006  - Chunying Liu modified the code to run at NCDC 

c*
c* INPUTS
c*  passed interactively or by script
c*    	iydo .......... user-specified year to do
c*	mondo ......... user-specified month to do
c*  land mask (land = 0, sea = 1)
c*      unit 30 ........ $ersst_DAT_DIR/input/static/mask2d.dat
c*  OI anomaly and counts used to compute std dev for quality screening
c*	unit 21 ....... $ersst_DAT_DIR/input/static/oiv2.ano2.1982.2002
c*  Set of 130 EOTs for 2 deg grid
c*	unit 21 ....... $ersst_DAT_DIR/input/static/eo6.ev130.ano.dat
c*  updated results of LF analysis                     
c*	unit 21 ....... $ersst_DAT_DIR/inter/analysis/lfsst.1978.situ.last.v3b.dat
c*  in situ SST updated anomalies; yyyy updated by program  
c* 	unit 21 ........ $ersst_DAT_DIR/inter/situmerg/ssta.merg.situ.mon.v3b.yyyy.dat
c*
c* OUTPUTS
c*  updated HF anomaly weights, 1985-latest month
c* 	unit 51......$ersst_DAT_DIR/inter/analysis/hfsst.1985.situ.last.wgt.v3b.dat
c*
c* PARAMETERS
c*	im, jm	= x- and y-size of 2 deg grid
c*	nsm	= number of "sea" pixels in 2-deg grid
c*	nmo	= total number of EOT modes
c*	crit	= critical value below which data is not used in regression
c*	fs 	= screening factor (multiplier for std dev for quality test)
c*	scm	= minimum std dev range accepted
c*
c* VARIABLES
c*   Main arrays on the 2 deg grid
c*	alf	= OI anomaly; then used to sum then average anomalies (std dev computation)
c*		= updated LF anomalies
c*	ac	= OI counts
c*		= used to average squares of anomalies (std dev computation)
c*	cn
c*	sdf
c*	blf	= updated lf anomalies
c*	mask	= 2-deg land mask
c*  vectors with nsm elements (number of sea pixels)
c*	cj	= area (cosine latitude) weights for sea pixels
c*  array containing number of modes by number of sea pixels
c*	gx	= EOT modes
c*  vector with elements y number of sea pixels by 3 (time window)
c* 	wt	= wieghts from fit
c*	dat3	=
c*
c*
c* SUBPROGRAMS
c*  scrftl
c*  lsfit
c*  ludcmp
c*  lubksb
c*  $ersst_LIB_DIR/maxmin.f
c*
c* LANGUAGE
c*  Fortran
c*
c@*****
c
      parameter(im=180,jm=89,nsm=11074)
      parameter(nmo=130,crit=0.20,fs=4.5,scm=1.0)
      real a2(im,jm),alf(im,jm),ac(im,jm),cn(im,jm),sdf(im,jm)
      real cj(nsm),ft(nsm),dat(nsm),gx(nsm,nmo),wt(nmo),dat3(nsm,3)
c**NEW Mar 20 2007****************************************
      real blf(im,jm)
c*********************************************************
      integer mask(im,jm)
      character fm*11,cyr*4,st*3,pd1*18
      CHARACTER(LEN=80) :: Dir_data,fname
      data fm/'unformatted'/, st/'old'/

      undef=-999.9
      print*, 'Input the year to do:'
      read(5,*) iydo
      print*, 'Input the month to do:'
      read(5,*) mondo

      CALL getenv("ersst_DAT_DIR", Dir_data)

      fname=adjustl(adjustr(Dir_data)//
     & '/inter/analysis/hfsst.1985.situ.last.wgt.v3b.dat')
      open(51,form=fm,iostat=ierr,file=fname)
      IF (ierr.NE.0) PRINT*,'Error opening: '//fname

c* Read in the 2-d mask (land = 0, sea = 1).
     
      fname=adjustl(adjustr(Dir_data)//
     &  '/input/static/mask2d.dat')
      open(30,form=fm,iostat=ierr,file=fname)
      IF (ierr.NE.0) PRINT*,'Error opening: '//fname
      read(30) mask
      close(30)
c
c* Compute the OI S.D. for checking, multiply it by fs.

      fname=adjustl(adjustr(Dir_data)//
     & '/input/static/oiv2.ano2.1982.2002')
      open(21,form=fm,iostat=ierr,file=fname)
      IF (ierr.NE.0) PRINT*,'Error opening: '//fname

      do 3 j=1,jm
      do 3 i=1,im
       alf(i,j)=0.0
       ac(i,j) =0.0
 3    continue
      rnt=real(12*(2003-1982))
      do iy=1982,2002
       do mon=1,12
        read(21) a2
        do 4 j=1,jm
        do 4 i=1,im
         alf(i,j)=alf(i,j)+a2(i,j)/rnt
         ac(i,j) =ac(i,j)+a2(i,j)*a2(i,j)/rnt
 4      continue
       end do
      end do
      do 6 j=1,jm
      do 6 i=1,im
       sdf(i,j)=1.0
       if(mask(i,j).eq.1)then
        vv=ac(i,j)-alf(i,j)*alf(i,j)
        if(vv.lt.0.0) vv=0.0001
        sdf(i,j)=fs*sqrt(vv)
        if(sdf(i,j).lt.scm) sdf(i,j)=scm
       endif
 6    continue
      close(21)

c
c* Read in the modes.
      fname=adjustl(adjustr(Dir_data)//
     & '/input/static/eot6.ev130.ano.dat')
      open(21,form=fm,iostat=ierr,file=fname)
      IF (ierr.NE.0) PRINT*,'Error opening: '//fname

      do nm=1,nmo
       read(21,end=102,err=102) a2
c       print*,'-----eot6-----',nm,a2(179,44),a2(179,45)
c       call maxmin(a2,im,jm,undef,'a22     ')
       kn=0
       do 7 j=1,jm
       do 7 i=1,im
        if(a2(i,j).gt.-990.)then
         kn=kn+1
         gx(kn,nm)=a2(i,j)
        endif
7      continue
c       print*, 'kn111=',kn
       if(kn.ne.nsm) stop 'nsm match'
      end do
      close(21)
      
c* Get the cosine of the latitude weights and a land-sea mask.
      kn=0
      do 8 j=1,jm
      rlat=0.017453293*real(2*j-90)
      cj1=cos(rlat)
      do 8 i=1,im
cliu       mask(i,j)=0
       if(a2(i,j).gt.-990.)then
        kn=kn+1
        cj(kn)=cj1
cliu        mask(i,j)=1
       endif
8     continue
 
      print*, 'kn=',kn
c
c* Compute the weights for the recentered anomaly.
      fname=adjustl(adjustr(Dir_data)//
     & '/inter/analysis/lfsst.1978.situ.last.v3b.dat')
      open(21,form=fm,iostat=ierr,file=fname)
      IF (ierr.NE.0) PRINT*,'Error opening: '//fname

      do iy=1978,1984
       read(21,end=101,err=101) alf
      end do
      
c* Initalize the dat3 array.
      do k=1,3
       do ns=1,nsm
        dat3(ns,k)=-999.9
       end do
      end do

c* Loop over years.
c**NEW Mar 20 2007****************************************
      knlf=0
c*********************************************************
      do 95 iy=1985,iydo
        write(cyr,'(i4)') iy
        fname=adjustl(adjustr(Dir_data)//
     &    '/inter/situmerg/ssta.merg.situ.mon.v3b.'//cyr//'.dat')
        open(23,form=fm,iostat=ierr,file=fname)
        IF (ierr.NE.0) PRINT*,'Error opening: '//fname

c**NEW Mar 20 2007****************************************
        read(21,end=10,err=10) blf
c        print*,'-----alf-----',iy,blf(179,44),blf(179,45)
        do j=1,jm
          do i=1,im
              alf(i,j)=blf(i,j)
          end do
        end do
        knlf=knlf+1
 10     if(knlf.eq.0)then
           print*, 'Problem with LF file, stopping'
           go to 100
        endif
        amxlf=-1.e9
        amnlf=-amxlf
        do j=1,jm
           do i=1,im
              if(alf(i,j).lt.amnlf) amnlf=alf(i,j)
              if(alf(i,j).gt.amxlf) amxlf=alf(i,j)
           end do
        end do
        print*, 'LF:',iy,knlf,amnlf,amxlf
c*********************************************************
       
        amax=-1.e9
        amin=-amax
        mme=12
        if(iy.eq.iydo)mme=mondo
        do 90 mon=1,mme
c           print*,'year=',iy,'month=',mon
c* Get the raw recentered anomaly.
           read(23,end=103,err=103) a2
	   read(23,end=104,err=104) cn
c        call maxmin(a2,im,jm,undef,'a2      ')
c	call maxmin(cn,im,jm,undef,'cn      ')

	   do k=1,2
              kp=k+1
              do ns=1,nsm
                 dat3(ns,k)=dat3(ns,kp)
              end do
           end do

           kn=0
           do 5 j=1,jm
           do 5 i=1,im
              if(mask(i,j).eq.1)then
                 kn=kn+1	  
                 tmp=-999.9
c          if(a2(i,j).gt.-990.)then
                 if(cn(i,j).gt.0.001)then
                    tmp=a2(i,j) - alf(i,j)
	            if(abs(tmp).gt.sdf(i,j))tmp=-999.9
cliu           if(abs(tmp).gt.8.0) tmp=-999.9
                 endif
cliu          dat(kn)=tmp
	         dat3(kn,3)=tmp
	  
               endif
 5          continue
 
c* Skip to get another month for the first month.
            if(iy.eq.1985)then
               if(mon.eq.1) go to 90
            endif
	
c* Set up the data for the month.  Use the center value if it is
c* defined.  Otherwise estimate the value from surrounding months if
c* they are defined.
           do ns=1,nsm
              dat(ns)=-999.9
              if(dat3(ns,2).gt.-990.)then
                 dat(ns)=dat3(ns,2)
              else
                 s1=0.0
                 s2=0.0
                 if(dat3(ns,1).gt.-990.)then
                    s1=s1+dat3(ns,1)
                    s2=s2+1.0
                 endif
                 if(dat3(ns,3).gt.-990.)then
                    s1=s1+dat3(ns,3)
                    s2=s2+1.0
                 endif
                 if(s2.gt.0.0) dat(ns)=s1/s2
              endif
           end do	 
 
c        call maxmin(dat,nsm,1,undef,'dat     ')
c
c* Compute the weights for the increment (wt) and the 1st guess (wr),
c* where the fitted anomaly is computed by
c*   anom(x) = sum, m=1,nmo, { gx(x,m)*(wt(m) }  + alf(x).
           call scrft1(gx,dat,nsm,nmo,cj,crit,nfit,ft,wt)
           monw=mon-1
           iyw=iy
           if(monw.lt.1)then
              monw=12
              iyw=iy-1
           endif
           write(51) monw,iyw,nfit,wt
	
cliu        write(82,91) mon,iy,nfit
c           write(*,91) monw,iyw,nfit
           do n=1,nsm
              if(ft(n).lt.amin) amin=ft(n)
               if(ft(n).gt.amax) amax=ft(n)
           end do
 90     continue
        close(23)
        write(6,93)cyr,amin,amax
 95   continue
 
 
 
c* Finish up the last month, using it and the previous month's data.
      do ns=1,nsm
        dat(ns)=-999.9
        if(dat3(ns,3).gt.-990.)then
           dat(ns)=dat3(ns,3)
        else
           if(dat3(ns,2).gt.-990.) dat(ns)=dat3(ns,2)
        endif
      end do
      print*,dat(1)
      call scrft1(gx,dat,nsm,nmo,cj,crit,nfit,ft,wt)


      monw=monw+1
      iyw=iyw
      if(monw .eq.13)then
         monw=1
         iyw=iyw+1
      endif


      write(51) monw,iyw,nfit,wt
      write(*,91) monw,iyw,nfit
 
      print*, 'GREEN-SUCCESSFUL! hfsst.situ.v3b.f'
      call exit(1)
 100  print*, 'RED-STOP! 100 hfsst.situ.v3b.f data problem'
      call exit(100)
 101  print*, 'RED-STOP! 101 lfsst.1978.situ.last.v3b.dat problem'
      call exit(100)
      
 102  print*, 'RED-STOP! 102 lfsst.1978.situ.last.v3b.dat problem'
      call exit(100)
      
 103  print*, 'RED-STOP! 103 lfsst.1978.situ.last.v3b.dat problem'
      call exit(100)
 104  print*, 'RED-STOP! 104 lfsst.1978.situ.last.v3b.dat problem'
      call exit(100)
      
 91   format(3i5)
 93   format('Finished ',a4,', range = ',f6.2,' to ',f6.2,'.')
      end
c==================END MAIN PROGRAM=========================
c@***s*  ERSST/hfsst.situ.v3b.f/scrftl
c* NAME
c*	scrft1
c*
c* LOCATION
c*	inside $ersst_SRC_DIR/hfsst.situ.v3b.f
c*
c* WARNING 
c*
c* PURPOSE
c* To get the best fit using forward stepwise screening regression.
c*
c* DESCRIPTION
c* For each mode, the data is screened to determine if the mode is 
c* adaquately supported by spatial sampling.  This is done by 
c* computing what fraction of mode's variance is accounted for
c* (Smith and Reynolds 2003 Appendix B).  First, modes with <3 data 
c* values are excluded. Only EOT modes supported with fraction 
c* above a critical value (0.2) are used for the fit. The valid data
c* is passed to subroutine lsfit that performs the regression, and 
c* returns the computed weights for each mode.  The full fitted value
c* is then computed, and returned to the main program.
c*
c* AUTHOR
c*  Thomas M. Smith
c*
c* CREATION DATE
c*  December 2006
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  2009 Sep -Viva Banzon added comments and put into robodoc format
c*
c* INPUTS
c*          nspace......Number of spatial points
c*          max.........Maximum number of modes to use
c*          crit........critical fraction var of mode supported
c*          dat().......1-D Array of data (anomalies)
c*          cj()........1-D array relative area weights
c* 	     gx(,).......2-D Array of spatial modes
c*
c* RETURNED VALUE
c*	     ft().......On output this has the best fit
c*          wt()........weights for each mode (-999.9 if not used).
c*          nfit........Number of modes used in the final fit.
c*          To reconstruct later sum over max: gx(n,m)*(wt(m)).
c*
c*
c* PARAMETERS 
c*	 nsmax...maximum possible number of spatial points (> 11074 used)
c*      maxx....maximum possible number of modes (> 130 used) 
c*
c* VARIABLES
c*	e2c....area weighted square of EOT mode
c*	sum1...sum of weighted EOT modes squared for all sea gridpoints
c*	sum2...sum of weighted EOT modes squared for gridpoint with data
c*	kn.....number of data available for EOT mode
c*	wf.....weights of fit for each EOT returned by lsfit
c*	gxf....2-D array of spatial modes  passed on to lsfit
c*
c* TODO
c*  Subroutine may be moved out of the MAIN.
c*
c* SUBPROGRAMS
c*  lsfit
c*  
c* LANGUAGE
c*  Fortran 
c*
c@*****
c
      subroutine scrft1(gx,dat,nspace,max,cj,crit,nfit,ft,wt)
      parameter (nsmax=11566,maxx=200)
      real wt(*),gx(nspace,*),wf(maxx),gxf(nsmax,maxx),cj(*)
      real dat(*),ft(*)
      integer iuse(maxx)
      do m=1,max
       iuse(m)=0
       wt(m)=-999.9
      end do
c* Check to see if a mode is adaquately supported by sampling.
      nfit=0
      do m=1,max
       sum1=0.0
       sum2=0.0
       kn=0
       do n=1,nspace
        e2c=gx(n,m)*gx(n,m)*cj(n)
        sum1=sum1+e2c
        if(dat(n).gt.-990.)then
         sum2=sum2+e2c
         kn=kn+1
        endif
       end do
       fva=sum2/sum1
       if(kn.lt.3)fva=0.0
       if(fva.gt.crit)then
        nfit=nfit+1
        do n=1,nspace
         gxf(n,nfit)=gx(n,m)
        end do
        iuse(m)=1
       end if
      end do
c* Get the fit, return the weights and full fit.
      if(nfit.gt.0)then
       call lsfit(gxf,dat,nsmax,nspace,nfit,cj,wf)
       n=0
       do m=1,max
        if(iuse(m).ne.0)then
         n=n+1
         wt(m)=wf(n)
        endif
       end do
      endif
      do i=1,nspace
       sum=0.0
       do mode=1,max
        if(wt(mode).gt.-990.) sum = sum + gx(i,mode)*wt(mode)
       end do
       ft(i) = sum
      end do
      return
      end
c----------------------END OF SUBROUTINE--------------------------
c@***s*  ERSST/hfsst.situ.v3b.f/lsfit
c* NAME
c*	lsfit
c*
c* LOCATION
c*	inside $ersst_SRC_DIR/hfsst.situ.v3b.f
c*
c* WARNING 
c* variables
c*
c* PURPOSE
c* To get the least-squared weights for observed data fitted to 
c* the given number of spatial EOT patterns. 
c*
c* DESCRIPTION
c* The pre-screened data is used as input for the regression of data
c* to the EOT modes.  The system of linear regression equations is 
c* is treated as a matrix relation: A w = b, solved by using known
c* methods, LU decomposition and backward substitution, by calls to
c* the subroutines ludcmp and lubksb.
c*
c* AUTHOR
c*  Thomas M. Smith
c*
c* CREATION DATE
c*  December 2006
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  2009 Sep -Viva Banzon added comments and put into robodoc format
c*
c* INPUTS
c*	cj	= area weights for 2 deg grid
c* 	gx	= EOT modes (used to form matrix A)
c* 	tsinc	= data (goes into vector b)
c* 	nspace = 11566 (maximum number of sea values allowed)
c* 	lspace = 11074 (number of ocean "pixels"; passed from scrfit1)
c*
c* RETURNED VALUE
c*	wt	= weight for EOT modes (unknown vector w)
c*
c* PARAMETERS 
c*	maxx	= maximum number of EOT modes allowed	
c*
c* VARIABLES
c*	a	= elements of matrix A (weighted cross product of input)
c*	b	= solution (fitted weights for each EOT mode)
c*	indx	=
c*
c* TODO
c*	Put in separate subdirectory from main program?
c*
c* SUBPROGRAMS
c*  ludcmp
c*  lubksb
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
      subroutine lsfit(gx,tsinc,nspace,lspace,max,cj,wt)
      parameter(maxx=200)
      real tsinc(*),gx(nspace,*),wt(*),cj(*)
      double precision a(maxx,maxx),b(maxx)
      integer indx(maxx)
c* Form matrix A
      do 4 i=1,max
      do 4 j=1,max
       a(i,j) = 0.
       do k=1,lspace
        if(tsinc(k).gt.-990.) a(i,j)=a(i,j)+gx(k,i)*gx(k,j)*cj(k)
       end do
 4    continue
c* Form array x
      do i=1,max
       b(i) = 0.
       do k=1,lspace
       	if(tsinc(k).gt.-990.)b(i)=b(i)+tsinc(k)*gx(k,i)*cj(k)
       end do
      end do
c* Get the LU decomposition of A and solve the linear system AX=B.
c* The solution X is returned in B.
      call ludcmp(a,max,maxx,indx,d)
      call lubksb(a,max,maxx,indx,b)
      do i=1,max
       wt(i)=b(i)
      end do
      return
      end
c
c-----------------------END SUBROUTINE------------------------------
c@***s*  ERSST/hfsst.situ.v3b.f/ludcmp
c* NAME
c*	ludcmp
c*
c* LOCATION
c*	inside $ersst_SRC_DIR/hfsst.situ.v3b.f
c*
c* WARNING 
c*
c* PURPOSE
c* This does an LU decomposition on the matrix A, size n x n.  The
c* output is returned in A.  This is Crout's method with partial
c* pivoting, described in Numerical Recepies, sec 2.3.
c*
c* DESCRIPTION
c* The LU decomposition with partial pivoting has the form:
c* A = LUP where L has only zeroes above the diagonal (lower triangular)
c*               U has only zeroes below the diagonal (upper triangular)
c*               P is a permutation matrix (i.e.,
c*                  contains 1s and 0s with only one 1 per column)
c* However, in this case, the solution is done "in place"
c* Thus, A will also contain the solution (see below)
c*
c* Comments from Numerical Recipes:
c* Given a matrix a(1:n, 1:n) with physical dimension np by np, this routine
c* replaces a by the LU decomposition of a row-wise premutation of itself.
c* The solution U and L values can be placed in the input matrix because those
c* values will not be used anymore once a solution is determined for that element
c* and the replacement element is needed for the solution of the next element.
c*
c* AUTHOR
c*  Thomas M. Smith
c*
c* CREATION DATE
c*  December 2006
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  2009 Sep -Viva Banzon added comments and put into robodoc format
c*
c* INPUTS
c*	a 	= input matrix
c*	n	= size of matrix side
c*
c* RETURNED VALUE
c*	a 	= arrangment: upper triangular matrix values up to the diagonal,
c*		  and the lower triangular matrix values below the diagonal)
c* 	indx 	= vector that records the permutation effected by partial pivoting
c*	d	= +/-1 depending on whether # of row interchanges was even or odd
c*
c*
c* PARAMETERS 
c* 	nmax	= 200 is Largest expected n 
c* 	tiny	= 1.0e-20) is a very small number
c*
c* VARIABLES
c*	aamax	= minimum value for row
c*	vv	= scaling for row
c*
c* TODO
c*	Put in separate subdirectory from main program?
c*
c* SUBPROGRAMS
c*  none
c*  
c* LANGUAGE
c*  Fortran 
c@*****


      subroutine ludcmp(a,n,np,indx,d)
      parameter (nmax=200,tiny=1.0e-20)
      dimension indx(*),vv(nmax)
      double precision a(np,np)
      d=1.
      do 12 i=1,n
       aamax=0.
       do 11 j=1,n
        if(abs(a(i,j)).gt.aamax) aamax=abs(a(i,j))
 11    continue
       if (aamax.eq.0.) pause 'singular matrix.'
       vv(i)=1./aamax
 12   continue
      do 19 j=1,n
       if (j.gt.1) then
        do 14 i=1,j-1
         sum=a(i,j)
         if(i.gt.1)then
          do 13 k=1,i-1
           sum=sum-a(i,k)*a(k,j)
 13       continue
          a(i,j)=sum
         endif
 14     continue
       endif
       aamax=0.
       do 16 i=j,n
        sum=a(i,j)
        if(j.gt.1)then
         do 15 k=1,j-1
          sum=sum-a(i,k)*a(k,j)
 15      continue
         a(i,j)=sum
        endif
        dum=vv(i)*abs(sum)
        if(dum.ge.aamax) then
         imax=i
         aamax=dum
        endif
 16    continue
       if(j.ne.imax)then
        do 17 k=1,n
         dum=a(imax,k)
         a(imax,k)=a(j,k)
         a(j,k)=dum
 17     continue
        d=-d
        vv(imax)=vv(j)
       endif
       indx(j)=imax
       if(j.ne.n)then
        if(a(j,j).eq.0.)a(j,j)=tiny
         dum=1./a(j,j)
         do 18 i=j+1,n
          a(i,j)=a(i,j)*dum
 18      continue
       endif
 19   continue
      if(a(n,n).eq.0.)a(n,n)=tiny
      return
      end
c
c---------------------------END OF SUBROUTINE-----------------------
c@***s*  ERSST/hfsst.situ.v3b.f/lubksp
c* NAME
c*	lubksp
c*
c* LOCATION
c*	inside $ersst_SRC_DIR/hfsst.situ.v3b.f
c*
c* WARNING 
c*
c* PURPOSE
c* This solves the system Ax=b, where here A is the LU decomposition of
c* the matrix A that was passed into ludcmp()
c*
c* DESCRIPTION
c* This solves the system Ax=b, where here A is the LU decomposition of
c* the original matrix A that was passed into ludcmp()
c* COMMENTS from Numerical Recipes:
c* Implements forward and backward substitution
c* Solves the set on n linear equations A . X = B
c*
c* AUTHOR
c*  Thomas M. Smith
c*
c* CREATION DATE
c*  December 2006
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  2009 Sep -Viva Banzon added comments and put into robodoc format
c*
c* INPUTS
c*	a 	= the LU decomposition of A
c*	indx	= the permutation vector
c*	b(1:n)	= the right-hand side of vector B
c*	n	= number of EOT modes
c*	np	= maximum number of EOT modes allowed
c*	indx	= vector that records the permutation effected by partial pivoting
c*			from subroutine ludcmp
c*
c* RETURNED VALUE
c*	b	= the solution vector X
c*
c* PARAMETERS 
c*	None
c*
c* VARIABLES
c*	ll	= index of current b
c*
c* TODO
c*
c* LANGUAGE
c*  Fortran 
c@*****

      subroutine lubksb(a,n,np,indx,b)
      dimension indx(*)
      double precision a(np,np),b(*)
      ii=0
      do 12 i=1,n
       ll=indx(i)
       sum=b(ll)
       b(ll)=b(i)
       if(ii.ne.0)then
        do 11 j=ii,i-1
         sum=sum-a(i,j)*b(j)
 11     continue
       else if(sum.ne.0.) then
        ii=i
       endif
       b(i)=sum
 12   continue
      do 14 i=n,1,-1
       sum=b(i)
       if(i.lt.n)then
        do 13 j=i+1,n
         sum=sum-a(i,j)*b(j)
 13     continue
       endif
       b(i)=sum/a(i,i)
 14   continue
      return
      end
      
      include 'maxmin.f'
