c        1         2         3         4         5         6         7
c23456789012345678901234567890123456789012345678901234567890123456789012
c     7                                                               72
c     |                                                                |
c@***h*  ERSST/monice1d-med-oper.f
c*
c* NAME
c*  monice1d-med-oper.f
c*
c* LOCATION
c*  $ersst_SRC_DIR/monice1d-med-oper.f
c*
c* WARNING
c*  Change destination directories if not running operationally
c*  Does not issue explicit I/O error messages
c*
c* PURPOSE
c*  To generate monthly 1 deg ice concentrations from 1/4 deg daily median ice.
c*
c* DESCRIPTION
c*  Daily median ice concentrations on a 1/4 deg grid from OI processing
c*  are averaged to produce the 1-deg monthly fields 
c*  The GSFC sea ice dataset is used from 1985 to 2004.  Then the program
c*  automatically switches from GSFC to NCEP ice in 2005
c*
c* AUTHOR
c*  Thomas M Smith
c*
c* CREATION DATE
c*  unknown
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  03/03/2010 - C. Liu hardcoding
c*  08/28/2009 - V. Banzon  Added comments in robodoc-compatible format
c*  07/04/2008 - C. Liu commented out error message for limited ice data
c*  July 2006  - Chunying Liu modified the code to run at NCDC 
C*
c* INPUTS
c*	iyrs,imos,idas ..... year month day to begin processing
c*	iyre,imoe,idae ..... year month day to end processing
c* Note that the series of ?  are placeholders that the program loop fills in with 
c* year (4 digits), month (01-12), and day (01-31), as needed
c*	unit 1 .....before 2005: $ersst_DAT_DIR/input/ice/gsfc/cice.YYYYMMDD 
c*		soft linked to /raid2/sstday/????/obs/gsfc-med/cice.???????? 
c*  	unit 2 ..... after 2005: $ersst_DAT_DIR/input/ice/ncep/icecon.YYYYMMDD
c*              soft linked to /raid2/sstday/????/grads/ice-med/icecon.????????
c*
c* OUTPUTS
c* Note that the yyyy are placeholders that are filled in with year to be processed.
c* Thus the output filename changes each year processed, but the file contains all 12 months of output
c*	unit 61 ....... $ersst_DAT_DIR/inter/ice1d/medmonice.ave-1d.yyyy

c* PARAMETERS
c*  imx, imy	= x and ysize of quarter deg grid
c*  imx1,imy1	= x and ysize of 1 deg grid
c*
c* VARIABLES
c*  icemed	= 2-element vector holding filepath/name for GSFC and NCEP input
c*  ivs, ive	= fixed positions in input filename string to begin/end update of year month day 
c* 		  (2-element vector since values different for GSFC or NCEP)
c*  cidat	= daily 1/4 deg sea ice data
c*  cimon	= contains monthly sum of daily data, then once out of month loop, the average
c*  ctmon	= counts for monthly sums
c*
c* SUBPROGRAMS
c*  maxmin
c*
c* TODO
c*  the maxmin subroutine should be put in a subroutine directory and replaced by an include
c*  could be updated to fit Fortran 90 improvements, e.g., array operations
c*  Fortran now has max and min functions so the subroutine could be replaced
c*
c* LANGUAGE
c*  Fortran 77 modified for 90 compatibility
c*
c@****
c*===================================END OF HEADER================================
c
      PARAMETER (imx=1440,imy=720,imx1=360,imy1=180) 
      dimension cidat(imx,imy),ivs(2),ive(2)
      dimension cicmon(imx1,imy1),ctmon(imx1,imy1)
c
c    program goes from median ice directly to monthly average
c    automatically switched from GSFC to NCEP ice in 2005
c      
      character*55 icemed(2)
      character*4 ice_in(2),year
      character*100 outfil
      character*8  date, iname(2)

       CHARACTER(LEN=60) :: Dir_data
       
       data ice_in(1) /'gsfc'/
       data ice_in(2) /'ncep'/
       data iname(1) /'cice.'/
       data iname(2) /'icecon.'/

      logical lexist
c     
      
      CALL getenv("ersst_DAT_DIR", Dir_data)
c      print*, Dir_data
           

      read(5,*) iyrs
      read(5,*) imos
      read(5,*) idas
      read(5,*) iyre
      read(5,*) imoe
      read(5,*) idae
      
      print*,'The requested year month is', iyre,imoe 
      print*, 'Need to process from January of ', iyre            
c iyrn=2005 is the year shift to NCEP ice
! default indx=1 is for gsfc 
      iyrn = 2005
      indx = 1
      undef = -9999.
      
      print 201,iyrs,imos,idas
  201 format ('START: year =',i5,3x,'month =',i3,3x,'day =',i3)
      print 202,iyre,imoe,idae
  202 format (' END:  year =',i5,3x,'month =',i3,3x,'day =',i3)
c
      do 110 iyr=iyrs,iyre
         write (year,'(i4.4)') iyr
         print*,year

         if (iyr .ge. iyrn) then
           print *,'switching from gsfc to ncep ice'
	   indx = 2
         endif
!one ice file per year
!current year is overwritten      
         outfil=adjustl(adjustr(Dir_data)//
     &      '/inter/ice1d/medmonice.ave-1d.'//year)
         print *,'median ice: open =',outfil
         open (61,file=outfil,form='unformatted')      

       
         imons = 1
         imone = 12      
         if(iyr .eq. iyre)imone=imoe
         if(iyr .eq. iyrs)imons=imos
            
         do 120 imo=imons,imone     

            do i=1,imx1
               do j=1,imy1
	          cicmon(i,j) = 0.
	          ctmon(i,j) = 0.
	       enddo
            enddo
            ndays = 0
            do ida=1,31

! The filename is composed from the date and ice filename prefix
               write (date,'(i4.4,i2.2,i2.2)')  iyr,imo,ida
              icemed(indx)=adjustl(adjustr(Dir_data) // '/input/ice/'//
     2           ice_in(indx)//'/'//trim(iname(indx))//date)
	
c	      print*,'ice input file=', icemed(indx)
       
              INQUIRE(FILE=icemed(indx),EXIST=LEXIST)
!              print *,lexist
              if (lexist) then
              ndays = ndays + 1
	  
    	      open (31,file=icemed(indx),status='old',form='unformatted')
              READ (31,END=200) kyr,kmo,kda,cidat
          
              do i=1,imx1
	        ie = i*4
	        is = ie - 3
c	        print *,i,is,ie
                do j=1,imy1
	          je = j*4
	          js = je - 3
	          do ii=is,ie
	            do jj=js,je
	              if (cidat(ii,jj).gt.undef) then
	                cicmon(i,j) = cicmon(i,j) + cidat(ii,jj)
		        ctmon(i,j)  = ctmon(i,j)  + 1.
	              endif
	            enddo
                  enddo
	        enddo
              enddo
	    else
               print *,'missing input file:',icemed(indx)
  	    endif
          enddo
          do i=1,imx1
          do j=1,imy1
	     if (ctmon(i,j).gt.0.5) then
	       cicmon(i,j) = cicmon(i,j)/ctmon(i,j)
	     else
	       cicmon(i,j) = undef
	     endif
           enddo
      enddo
      max = ndays*4*4

!  In 1987 Dec only has 5 days ice        
      if (ndays.lt.20.and.iyr.ne.1985.and.imo.ne.12)then
        print *,'YELLOW-WARNING number of days <20 ndays =',ndays
 	call exit(100)
      endif

c      if (imo.eq.1) then
        print 109,kyr,kmo,kda,ndays,max
  109   format ('year =',i5,3x,'month =',i3,3x,'day =',i3,3x,
     1 'ndays =',i3,3x,'max =',i8)
        call  maxmin (cicmon,imx1,imy1,undef,'mon cice')
        call  maxmin (ctmon, imx1,imy1,undef,'ice ct  ')
c      endif
      write (61) kyr,kmo,ndays,cicmon
  120 continue
  110 continue
      print 109,kyr,kmo,kda,ndays,max
     
      print *,'GREEN-SUCCESSFUL! monice1d-med-oper.f'
      call exit(1)
  200 PRINT 302
  302 FORMAT('RED-STOP!PREMATURE END OF INPUT DATA monice1d-med-oper.f')
      call exit(100)
      end
      
c==============END OF MAIN PROGRAM=======================================
c      subroutine maxmin (fld,im,jm,undef,name)
c@***s* ERSST/monice1d-med-oper.f/maxmin
c*
c* NAME
c*  maxmin
c*
c* LOCATION
c*  inside $ersst_SRC_DIR/monice1d-med-oper.f
c* 
c* PURPOSE
c*   To compute maximum, minium and average of input field fld and print to screen
c*
c* DESCRIPTION 
c*  The minimum value holder is first set to a very large number and
c*  the maximum value holder is first set to a very small number.  Then 
c*  the current value replaces the value in the min holders if the current
c*  value is smaller.  The maximum is determined the same way. The data is
c*  summed and counted within the loop, then the average is computed upon
c*  exiting the loop. The max, min and average are displayed on the screen.
c*
c* AUTHOR
c*  Thomas M Smith
c*
c* CREATION DATE 
c*  Unknown
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  2009 Sept -Viva Banzon added comments and put into robodoc format
c*
c* INPUTS
c*	fld	= data to be examined for statistics
c*	im,jm	= x and y size of array
c*	undef	= code for missing data
c*	name	= name for input data field
c*
c* RETURNED VALUE
c*	none
c*
c* PARAMETERS
c*    im1, jm1 = xsize and ysize of 1 degree grid
c*    im2, jm2 = xsize and ysize of 2 degree grid
c*
c* VARIABLES
c*	fldmin	= minimum of field
c*	fldmax	= maximum of field
c*	fld 	= data field
c*	ave	= average of field
c*	inum	= number of valid data in field
c*
c* TODO
c*	Put this in a subroutine library as this is used in many programs
c*
c* LANGUAGE
c*  Fortran 
c*
c@***** 
      subroutine maxmin (fld,im,jm,undef,name)
c
c   compute maximum, minium and average of fld and print
c
      dimension fld(im,jm)
      character*8 name
      real*8 ave,r
      fldmin = 1.e30
      fldmax = -fldmin
      inum = 0
      ave = 0.
      do 90 i=1,im
      do 90 j=1,jm
      if (fld(i,j).gt.undef) then
        inum = inum + 1
        ave = ave + fld(i,j)
        if (fldmin.gt.fld(i,j))  fldmin = fld(i,j)
        if (fldmax.lt.fld(i,j))  fldmax = fld(i,j) 
      endif
   90 continue
      if (inum.gt.0) then
        r = 1./float(inum)
        ave = ave*r
      else
        fldmin = 0.
        fldmax = 0.
      endif
      print 88,ave,fldmin,fldmax,inum,name
   88 format (' ave =',f10.3,2x,'min =',f10.3,2x,
     1 'max =',f10.3,2x,'inum =',i8,2x,a8)
      return 
      end
