c@***s* ERSST/ersst_netcdf.situ.v3b.f/put_cdf_ersst_situ_v3b.f       
c*
c* NAME
c*	 put_cdf_ersst_situ_v3b.f       
c*
c* LOCATION
c*  /raid2/ersst/ftn/SST/
c* 
c* PURPOSE
c*   To write out data (daily SSTs) in NetCDF format 
c*
c* DESCRIPTION 
c*	Monthly data fields (SST, anomalies, and error)are passed from the main program
c*	and written out in NetCDF format to a file with name also passed from Main.
c*	Fortran NetCDF functions are used to format the output.  Scaling and other
c*	attributes are hardcoded in.
c*
c* AUTHOR
c*  Chunying Liu
c*
c* CREATION DATE 
c*  Unknown
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  2009 Sept -Viva Banzon added comments and put into robodoc format
c*
c* INPUTS
c*	fname	=path and filename of netCDF output
c*  Integerized data*100 in 2-Degree grids (dimensions are lon_len, lat_len):
c*	sst2d	=reconstructed SST on 2 deg grid
c*	anom2d	=ERSST anomaly on 2 deg grid
c*	err2d	=ERSSt error on 2 deg grid
c*  Dates in various formats:
c*	cnetdate =data date expressed as YYYY-MM-DD 
c*	sdate	  =reference date in YYYYMMDD format
c*	refdate  =reference date in YYYY-MM-DD format
c*	iday	  =data date expressed as julian day number from reference day
c*
c* OUTPUT
c*	writes out data in netCDF format to fname (from Main)
c*
c* PARAMETERS
c*   lat_len 	= 89 (2 deg grid xsize; no of latitude cells)
c*   lon_len 	= 180 (2 deg grid ysize; no of longitude cells)c* (zlev_len = 1)
c*   time_len	= 1 (time records in entire time series)
c*   time_nr 	= 1 (number of time records in data)
c*   fillval	=-999 (fill for missing/bad values)
c*
c* VARIABLES
c*  timeunit	=string stating days from reference date to put in netCDF attributes
c*
c*  Variable IDs as integers from nf_def_var :  sstID, latID, lonID, zlevID, timeID, errID, anomID
c*  Variable Dimension ID's from nf_def_att : latDimID, lonDimID, zlevDimID, timeDimID
c*
c*  Scaling parameters for data to write in attributes:					
c*	min	=minimum value for parameter range	(changes by parameter)
c*	max	=maximum value for parameter range	(changes by parameter)
c*	offset	=bias to add	
c*	scale	=slope to multiply data with
c*
c*  Control variables:
c*	iret	=value returned by netCDF functions 
c*
c*  Data coordinates:
c*	lat(lat_len)	=lat gridpoints for 2 deg grid
c*	lon(lon_len		=lon gridpoints 2 deg grid
c*	zlev(zlev_len)	=height level data
c*	time(time_nr)	=time expressed in days from reference date
c*
c*  4-element vectors containing 4-D array dimensions
c*	sstDimIDs	=SST field dimensions
c*	errDimIDs	=error field dimensions
c*	anomDimIDs	=anomaly field dimensions
c*  4-D arrays (lon_len, lat_len,zlev_len,time_nr):  
c*	sst	= sst data in 4-D array expected by netCDF
c*	anom	= anomaly data in 4-D array expected by netCDF
c*	err	= error data in 4-D array expected by netCDF
c*
c* TODO
c*
c* SUBPROGRAMS
c*   	/raid2/ersst/ftn/SST/check_err90.f
c*
c* LIBRARY
c*   	/home/cliu/netcdf/netcdf-3.6.0-p1
c*
c* FUNCTIONS
c*	nf90_create	 creates the netCDF file
c*	nf90_put_att	 defines attributes (global and parameter)in netCDF file
c*	nf90_def_dim	 defines variable dimensions  in netCDF file
c*	nf90_def_var	 defines variables in netCDF file
c*	nf90_enddef	 ends define mode in netCDF file
c*	nf90_put_var	 writes data array in netCDF file
c*
c* MODULES
c*	netcdf
c*	typesizes 
c*
c* LANGUAGE
c*    Fortran 
c*
c@*****

      subroutine put_cdf_ersst_situ_v3b(fname,sst2d,anom2d,err2d,
     2 cnetdate,sdate,refdate,iday)
     
c-----------------------------------------------------------------------
c     write out daily SSTs in netcdf format 
c  
      !include 'netcdf.inc' 
	use netcdf
      	use typesizes 
	
c*************** Dimention and variable declarations:
      character fname*100 , sdate*10, timeunit*33,refdate*99,cnetdate*10
c
      integer ncid, iret
      integer*2 fillval,min,max
      real offset, scale

c     Define variable ids as integers (output from nf_def_var later)
      integer  sstID
      integer  latID
      integer  lonID
      integer  zlevID
      integer  timeID
      integer  errID
      integer  anomID
      

C     Define variable dimension IDs (output from nf_def_dim later)
      integer  latDimID
      integer  lonDimID
      integer  zlevDimID
      integer  timeDimID

C     Define dimension type & lengths - specify here
      integer  lat_len
      integer  lon_len
      integer  zlev_len
      integer  time_len 	!time records in the whole dataset/time series
      integer  time_nr		!actual time record in this dataset)
      parameter (lat_len = 89)
      parameter (lon_len = 180)
      parameter (zlev_len = 1)
c      parameter (time_len = NF90_UNLIMITED) 
      parameter (time_len = 1)     
      parameter (time_nr = 1)
      parameter (fillval=-999)
      
C     Define variables shapes
      real lat(lat_len)
      real lon(lon_len)
      real zlev(zlev_len)
      integer  time(time_nr)
      integer  sstDimIDs(4)
      integer  errDimIDs(4)
      integer  anomDimIDs(4)
      
      
      integer*2 sst(lon_len, lat_len,zlev_len,time_nr)
      integer*2 anom(lon_len, lat_len,zlev_len,time_nr)
      integer*2 err(lon_len, lat_len,zlev_len,time_nr)
      
      
      integer*2 sst2d(lon_len, lat_len)
      integer*2 anom2d(lon_len, lat_len)
      integer*2 err2d(lon_len, lat_len)      

      data zlev/0./
      time=iday
      
      sst(:,:,1,1)=sst2d(:,:)
      anom(:,:,1,1)=anom2d(:,:)
      err(:,:,1,1)=err2d(:,:)
            
      
c********** print out for checking:	

        print*,'check if the value read right before put to NetCDF'
	do j=10,10
	write(*,101)(sst(i,j,1,1),i=80,85)
	write(*,101)(anom(i,j,1,1),i=80,85)
	write(*,101)(err(i,j,1,1),i=80,85)			
	enddo
101	format(1x,6i10)	

c*****************open the nc file ID: ***************************
      fname=TRIM(fname)
      print*,'output file = ',fname
      call check_err90( NF90_CREATE(fname,NF90_CLOBBER,ncid) )

c********************* write out the global attributes: ************
      iret = NF90_PUT_ATT(ncid,NF90_GLOBAL,'title',
     &      'ERSST V3b in situ only')  
      call check_err90(iret)
      
      iret = NF90_PUT_ATT(ncid,NF90_GLOBAL,'history',
     &     'Version 3b')
      call check_err90(iret)
      
      iret = NF90_PUT_ATT(ncid,NF90_GLOBAL,'creation_date',
     &     cnetdate)
      call check_err90(iret)
  
      
      iret = NF90_PUT_ATT(ncid,NF90_GLOBAL,'description',
     & 'Smith, T.M., and R.W. Reynolds, 2003: Extended Reconstruction of 
     & Global Sea Surface Temperatures Based on COADS Data (1854-1997). 
     &Journal of Climate, 16, 1495-1510. A vailable at http://www.ncdc.n
     &oaa.gov/img/climate/research/sst/ersst.pdf  Climatology is based o
     &n 1971-2000 OI.v2 SST, In situ data: ICOADS2.4 before 2006 and NCE
     &P in situ data from 2007 to present')
      
     
      call check_err90(iret)

 
      iret = NF90_PUT_ATT(ncid,NF90_GLOBAL,'source',
     &     'NOAA/National Climatic Data Center')
      call check_err90(iret)

      iret = NF90_PUT_ATT(ncid,NF90_GLOBAL,'contact',
     &     'Dick Reynolds, email: Richard.W.Reynolds@noaa.gov & Chunying
     & Liu, email: Chunying.liu@noaa.gov')
      call check_err90(iret)
     
      iret = NF90_PUT_ATT(ncid,NF90_GLOBAL,'Conventions',
     &     'CF-1.0')
      call check_err90(iret)  

c********************* end of global attributes: ***************************
              
C     call out dimension IDs w/ error checks:
      
      call check_err90( nf90_def_dim(ncid,'time',time_nr,timeDimID) )
      call check_err90( nf90_def_dim(ncid,'zlev',zlev_len,zlevDimID))      
      call check_err90( nf90_def_dim(ncid,'lat',lat_len,latDimID) )
      call check_err90( nf90_def_dim(ncid,'lon',lon_len,lonDimID) )
           
C     call out variables IDs & assign attributes:
      !call put variable IDs:
      iret = nf90_def_var(ncid, 'time',NF90_REAL,timeDimID,timeID)
      					call check_err90(iret)
      iret = nf90_def_var(ncid,'zlev',NF90_REAL,zlevDimID,zlevID)
      					call check_err90(iret)					
      iret = nf90_def_var(ncid,'lat',NF90_REAL,latDimID,latID)
      					call check_err90(iret)
      iret = nf90_def_var(ncid,'lon',NF90_REAL,lonDimID,lonID)
      					call check_err90(iret)           
					
      					sstDimIDs(4) = timeDimID
					sstDimIDs(3) = zlevDimID
      					sstDimIDs(2) = latDimID
      					sstDimIDs(1) = lonDimID
      iret = nf90_def_var( ncid,'sst',NF90_SHORT,sstDimIDs,sstID)
					call check_err90(iret)
					
					anomDimIDs(4) = timeDimID
					anomDimIDs(3) = zlevDimID					
					anomDimIDs(2) = latDimID
      					anomDimIDs(1) = lonDimID
      iret = nf90_def_var(ncid,'anom',NF90_SHORT,anomDimIDs,anomID)
					call check_err90(iret)	
					
					errDimIDs(4) = timeDimID
					errDimIDs(3) = zlevDimID
					errDimIDs(2) = latDimID
      					errDimIDs(1) = lonDimID
      iret = nf90_def_var(ncid,'err',NF90_SHORT,errDimIDs,errID)
					call check_err90(iret)
					
					
c******************************************************************      
      !Assign attributes:
      iret = nf90_put_att(ncid, latID, 'long_name','Latitude')
      iret = nf90_put_att(ncid, latID, 'units', 'degrees_north')
      					call check_err90(iret)    
      iret = nf90_put_att(ncid, latID, 'grids', 
     &             'Uniform grid from -88 to 88 by 2')
      					call check_err90(iret)    

      iret = nf90_put_att(ncid, lonID, 'long_name','Longitude')
      iret = nf90_put_att(ncid, lonID, 'units', 'degrees_east')
     					call check_err90(iret)
      iret = nf90_put_att(ncid, lonID, 'grids', 
     &             'Uniform grid from 0 to 358 by 2')
      					call check_err90(iret)    

      iret = nf90_put_att(ncid, zlevID, 'long_name', 
     &                              'Sea surface height')
     					call check_err90(iret)
      iret = nf90_put_att(ncid, zlevID, 'units', 'meters')
     					call check_err90(iret)
	
      iret = nf90_put_att(ncid, zlevID, 'actual_range', '0, 0')
     					call check_err90(iret)				

      iret = nf90_put_att(ncid, timeID, 'long_name',
     &                    'Center time of the day')
      print*,'sdate:',sdate
      timeunit='days since '//refdate
      iret = nf90_put_att(ncid, timeID, 'units',timeunit)
      					call check_err90(iret)
					
      iret = nf90_put_att(ncid, timeID, 'delta_t','0000-01-00')
      					call check_err90(iret)
					
      iret = nf90_put_att(ncid, timeID, 'avg_period','0000-01-00')
      					call check_err90(iret)									
					
      offset=0.0
      scale=0.01
      min=-300
      max=4500					
					
     			
      iret = nf90_put_att(ncid, sstID, 'long_name', 
     &   'Extended reconstructed sea surface temperature')
      					call check_err90(iret)
      iret = nf90_put_att(ncid, sstID, 'units', 'degrees_C')
      					call check_err90(iret)
      iret = nf90_put_att(ncid, sstID, '_FillValue', fillval)
      					call check_err90(iret)
      iret = nf90_put_att(ncid, sstID, 'add_offset', offset)
      					call check_err90(iret)
      iret = nf90_put_att(ncid, sstID, 'scale_factor', scale)
      					call check_err90(iret)
      iret = nf90_put_att(ncid, sstID, 'valid_min', min)
      					call check_err90(iret)
      iret = nf90_put_att(ncid, sstID, 'valid_max', max)
      					call check_err90(iret)
	
      min=-1200
      max=1200							
     			
      iret = nf90_put_att(ncid, anomID, 'long_name', 
     &   'Extended reconstructed SST anomalies')
      					call check_err90(iret)
      iret = nf90_put_att(ncid, anomID, 'units', 'degrees_C')
      					call check_err90(iret)
      iret = nf90_put_att(ncid, anomID, '_FillValue', fillval)
      					call check_err90(iret)
      iret = nf90_put_att(ncid, anomID, 'add_offset', offset)
      					call check_err90(iret)
      iret = nf90_put_att(ncid, anomID, 'scale_factor', scale)
      					call check_err90(iret)
      iret = nf90_put_att(ncid, anomID, 'valid_min', min)
      					call check_err90(iret)
      iret = nf90_put_att(ncid, anomID, 'valid_max', max)
      					call check_err90(iret)				
      min=0
      max=1000		
      				
      iret = nf90_put_att(ncid, errID, 'long_name', 
     &   'Estimated error standard deviation of analyzed_ersst')      
      					call check_err90(iret)
      iret = nf90_put_att(ncid, errID, 'units', 'degrees_C')
      					call check_err90(iret)
      iret = nf90_put_att(ncid, errID, '_FillValue', fillval)
      					call check_err90(iret)
      iret = nf90_put_att(ncid, errID, 'add_offset', offset)
      					call check_err90(iret)
      iret = nf90_put_att(ncid, errID, 'scale_factor', scale)				
					call check_err90(iret)
      iret = nf90_put_att(ncid, errID, 'valid_min', min)
      					call check_err90(iret)
      iret = nf90_put_att(ncid, errID, 'valid_max', max)
      					call check_err90(iret)				
					
   
C     leave define mode
      call check_err90( NF90_ENDDEF(ncid) )

C     store time:
c      time(1)=1
      call check_err90( nf90_put_var(ncid, timeID, time) )
  
C     store lat:
      do ic=1,lat_len
          lat(ic)= -88 + (ic-1)*2
      enddo
      call check_err90( nf90_put_var(ncid, latID, lat) )
    
C     store lon
      do ic=1,lon_len
          lon(ic)= 0 + (ic-1)*2
      enddo
      call check_err90( nf90_put_var(ncid, lonID, lon) )

C     store zlev
      call check_err90( nf90_put_var(ncid, zlevID, zlev) )

C     store sst
      call check_err90( nf90_put_var(ncid, sstID, sst) )
      call check_err90( nf90_put_var(ncid, anomID, anom) )
      call check_err90( nf90_put_var(ncid, errID, err) )

   
c*********************** close out the nc file:
      call check_err90( nf90_close(ncid) )	! close output file

      return
      end
