c    67  1         2         3         4         5         6         7 2
c23456789012345678901234567890123456789012345678901234567890123456789012
c     7                                                               72
c 
c@***h*  ERSST/ssta.merg.situ.v3b.f
c*
c* NAME 
c*  ssta.merg.situ.v3b.0.f
c*
c* LOCATION
c*  $ersst_SRC_DIR/ssta.merg.situ.v3b.f
c*
c* WARNING
c*  Change destination directories if not running operationally
c*  Does not issue explicit I/O error messages
c*
c* PURPOSE
c*  Computes 2-deg monthly SST anomalies from 2-deg gridded monthly averages for ship and buoy,
c*  taking into account the differences in reliability and no. observations between the datasets
c*
c* DESCRIPTION
c*  Reads the gridded (2 deg) COADS ship and buoy average SSTs and counts.
c*  Computes anomalies with respect to 30-yr climatology (1971-2000 on an even grid),
c*  then merges the ship and bias-adjusted buoy data (Reynolds and Smith 1994, J. Clim).
c*  The output 2-deg monthly super observations are weighted by both the
c*  number of observations in each super observation and by the noise
c*  of the data type.
c*
c* AUTHOR
c*  Thomas M Smith
c*
c* CREATION DATE
c*  12/01/2006
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  08/01/2009 - V. Banzon added comments
c* 
c* INPUTS
c*  2-deg monthly ship, buoy with cyr replaced by program
c*	unit 21 ........  $ersst_DAT_DIR/inter/situ/ship.avg2.mon.cyr.dat
c*	unit 22 ........  $ersst_DAT_DIR/inter/situ/buoy.avg2.mon.cyr.dat
c*  2-deg climatology (even gridpoints,1971-2000)
c*	unit 21 ........ $ersst_DAT_DIR/input/static/clim.even.1971.2000.dat
c*  mondo, iyrend ... the month and year to do
c*
c*
c* OUTPUTS
c*  One yearly output file, monthly avg and counts written sequentially.
c*  2-deg gridded bias-adjusted SST anomalies from ship and buoys
c* 	unit 61 ........ $ersst_DAT_DIR/inter/situmerg/ssta.merg.situ.mon.v3b.cyr.dat
c*
c* PARAMETERS
c*     im, jm	=x,y size of 2 deg grid
c*     abmx	= largest SST anomaly acceptable
c*
c* VARIABLES
c*     su1	= sum of avg weighted SSTs
c*     su2   	= counts of avg weighted SSTs
c*  2-D arrays (2 deg grids):
c*     a2	= in situ (ship or buoy) avg SST
c*		 reused to hold merged SSTs (weighted)
c*     co	= in situ data counts
c*     c2	= output weighted counts of merged data
c*  3-D arrays (2 deg grids, and data type):
c*     sa	= SST from buoy or ship
c*     sc	= counts of buoy ship data
c*
c* SUBPROGRAMS
c*  range
c*  maxmin
c*
c* TODO
c*  put range in subroutine library
c*  pass environment variables
c*
c* LANGUAGE
c*  Fortran
c*
c@*****
c=============================END OF HEADER======================================
c
      parameter(im=180,jm=89,abmx=8.0)
      real a2(im,jm),co(im,jm),clim(im,jm,12),c2(im,jm)
      real sa(im,jm,2),sc(im,jm,2),wt1(2)
      character fm*11,st*3,cyr*4,pd1*18
c      data pd1/'/raid2/drsst/data/'/
      data fm/'unformatted'/, st/'old'/
      CHARACTER(LEN=80) :: Dir_data,fname
      
      CALL getenv("ersst_DAT_DIR", Dir_data)

c* Relative weights for ships, buoys (based on error
c* estimates from Reynolds and Smith 1994, J. Clim).
      data wt1/1.0,6.8/
c
      print*, 'Input the year to do:'
      read(5,*) iyrend
      print*, 'Input the month to do:'
      read(5,*) imoend
      print*, 'updated year month:',iyrend,imoend
      
      
c* Read in the climatology.
      fname=adjustl(adjustr(Dir_data)//
     &'/input/static/clim.even.1971.2000.dat')	
      open(21,status=st,form=fm,iostat=ierr,file=fname)
      print*,fname, ierr
      do mon=1,12
       read(21,end=100,err=100) a2
c       print*,mon
       do 1 j=1,jm
       do 1 i=1,im
        clim(i,j,mon)=a2(i,j)
 1     continue
      end do
      close(21)
c

c* Loop over the year.
      do 1000 iy=1985,iyrend
      mondo=12      
      if(iy.eq.iyrend)mondo=imoend
c
      write(cyr,'(i4)') iy
      fname=adjustl(adjustr(Dir_data)//
     &'/inter/situ/ship.avg2.mon.'//cyr//'.dat')
      open(21,status=st,form=fm,iostat=ierr,file=fname)
      print*,'Input :'//fname, ierr

      fname=adjustl(adjustr(Dir_data)//
     &'/inter/situ/buoy.avg2.mon.'//cyr//'.dat')
      open(22,status=st,form=fm,iostat=ierr,file=fname)
      print*,'Input :'//fname, ierr

      fname=adjustl(adjustr(Dir_data)//
     &'/inter/situmerg/ssta.merg.situ.mon.v3b.'//cyr//'.dat')
      open(61,form=fm,iostat=ierr,file=fname)
      print*,'Output :'//fname, ierr
c
c* Loop over months.
      do mon=1,mondo
c* Read in each data type and form anomalies.
       do kt=1,2
        iu=20+kt
c        print*,'iu=',iu

        read(iu,end=101,err=101) a2
        read(iu,end=101,err=101) co

        
	do 2 j=1,jm
        do 2 i=1,im
         sa(i,j,kt)=-999.9
         sc(i,j,kt)=co(i,j)
         if(a2(i,j).gt.-990.) sa(i,j,kt)=a2(i,j)-clim(i,j,mon)
         if(abs(sa(i,j,kt)).gt.abmx) sa(i,j,kt)=-999.9
         a2(i,j)=sa(i,j,kt)
 2      continue
c        call range(a2,amn,amx)
c        print*, kt,' Range: ',amn,amx
       end do
c* Merge the data.
       do 4 j=1,jm
       do 4 i=1,im
         su1=0.0
         su2=0.0
         do kt=1,2
          if(sa(i,j,kt).gt.-990.)then
            ww=sc(i,j,kt)*wt1(kt)
            su1=su1+sa(i,j,kt)*ww
            su2=su2+ww
          endif 
         end do
         a2(i,j)=-999.9
         if(su2.gt.0.0) a2(i,j)=su1/su2
	 c2(i,j)=su2

	
 4     continue
 
        
c* Write out the merged anomalies.
       if(iy.eq.iyrend)then
       print*,iy, mon
       call maxmin(a2,im,jm,undef,'a22     ')
       endif

       write(61) a2
       write(61) c2

c       call range(a2,amn,amx)
c       print*, kt,' Range: ',amn,amx
c       stop 'chk1'
c* End loop over months.

c      print*, 'year,month=',iy,mon
      end do
c
      print*, 'Finished merging for '//cyr//'.'
 1000 continue
      print*, 'GREEN-SUCCESSFUL! ssta.merg.situ.v3b.f'
      call exit(1)
 100  print*, 'RED-STOP! ssta.merg.situ.v3b.f data problem 100'
      call exit(100)
 101  print*, 'RED-STOP! ssta.merg.situ.v3b.f data problem 101'
      call exit(100)
 91   format(10i5)
      end
c
c*======================END MAIN PROGRAM======================
c@***s* ERSST/ssta.merg.situ.v3b.f/range
c* NAME
c*  range
c*
c* LOCATION
c*  inside $ersst_SRC_DIR/ssta.merg.situ.v3b_robo.f
c*
c* WARNING
c*  None
c*
c* PURPOSE
c* To compute the array rangefor 2 deg grid.
c*
c* DESCRIPTION
c* Minimum variable is initialized as a very large number and maximum
c* variable is set to a very small number.  Each value is compared to the min or
c* max, and replaces the value in the variable if it is smaller or larger, 
c* respectively.
c*
c* AUTHOR
c*  Thomas M Smith
c*
c* CREATION DATE 
c*  December 2006 (presumed)
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  2009 July -Viva Banzon added comments and put into robodoc format
c*
c* INPUTS
c*  a2		= array on 2 deg grid
c*
c* RETURNED VALUE
c*  amn	= minimum value of array
c*  amx	= maximum value of array
c*
c* PARAMETERS
c*  im, jm 	= xsize and ysize of 2 degree grid
c*
c* VARIABLES
c*  a2		= array on 2 deg grid
c*  i,j	= indices for 2-deg grid
c*
c* TODO
c*	separate out. this is exactly same subroutine in ice1t2
c*	also check if fortran 90 has a min max function
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****

      subroutine range(a2,amn,amx)
      parameter(im=180,jm=89)
      real a2(im,jm)
      amn=1.e9
      amx=-amn
      do 2 j=1,jm
      do 2 i=1,im
       if(a2(i,j).gt.-990.)then
        if(a2(i,j).lt.amn) amn=a2(i,j)
        if(a2(i,j).gt.amx) amx=a2(i,j)
       endif
 2    continue
      return
      end

      include 'maxmin.f'

